# Copyright (C) 2017-2026 Arun Sivaraman <arunsivaramanneo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import gi
import const
gi.require_version("Adw","1")
gi.require_version("Gtk","4.0")

from gi.repository import Adw, Gio, GObject, Gtk, Pango,Adw
from Common import create_scrollbar,fetchImageFromUrl,getLinkButtonImg,setMargin

Adw.init()

title = "About GPU-Viewer v3.32"

import gi

gi.require_version("Gtk", "4.0")
from gi.repository import Gtk, Gio, GObject  # noqa


class DataObject(GObject.GObject):
    def __init__(self, column: str):
        super(DataObject, self).__init__()
        self.column = column

def setup(widget, item):
    """Setup the widget to show in the Gtk.Listview"""
    label = Gtk.Label()
    label.props.use_markup = True
    label.props.xalign = 0.0
    label.set_ellipsize(Pango.EllipsizeMode.END)
    item.set_child(label)


def bind(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.column)

class AboutPage(Gtk.Box):
    def __init__(self):
        super().__init__(orientation=Gtk.Orientation.VERTICAL)
        self.grid = Gtk.Grid()
        
        title = "About GPU-Viewer v3.35"
        self.column_view = Gtk.ColumnView()
        self.column_view.props.show_row_separators = True
        self.column_view.props.single_click_activate = False
        self.column_view.props.show_column_separators = True
        
        factory = Gtk.SignalListItemFactory()
        factory.connect("setup", setup)
        factory.connect("bind", bind)
        
        scrollable_treelist1 = create_scrollbar(self.column_view)
        self.append(scrollable_treelist1)
        
        aboutColumn = Gtk.ColumnViewColumn.new(title)
        aboutColumn.set_resizable(True)
        aboutColumn.set_expand(True)
        aboutColumn.set_factory(factory)
        self.column_view.append_column(aboutColumn)
        
        selection = Gtk.SingleSelection()
        self.store = Gio.ListStore.new(DataObject)
        selection.set_model(self.store)
        self.column_view.set_model(selection)
        
        with open("../About_GPU_Viewer", "r") as file1:
            for i, line in enumerate(file1):
                self.store.append(DataObject(line.strip('\n')))
        
        self.append(self.grid)
        
        # Buttons with icons
        self.log_button = Gtk.LinkButton.new_with_label(const.CHANGE_LOG_LINK)
        self.log_button.set_tooltip_text(const.TOOLTIP_CHANGE_LOG)
        setMargin(self.log_button, 10, 5, 5)
        self.grid.attach(self.log_button, 0, 0, 1, 1)

        self.license_button = Gtk.LinkButton.new_with_label(const.LICENSE_HTML_LINK)
        self.license_button.set_tooltip_text(const.TOOLTIP_LICENSE)
        setMargin(self.license_button, 100, 5, 5)
        self.grid.attach_next_to(self.license_button, self.log_button, Gtk.PositionType.RIGHT, 1, 1)

        self.faq_button = Gtk.LinkButton.new_with_label(const.FAQ_LINK)
        self.faq_button.set_tooltip_text(const.TOOLTIP_FAQ)
        setMargin(self.faq_button, 100, 5, 5)
        self.grid.attach_next_to(self.faq_button, self.license_button, Gtk.PositionType.RIGHT, 1, 1)

        self.report_button = Gtk.LinkButton.new_with_label(const.ISSUE_LINK)
        self.report_button.set_tooltip_text(const.TOOLTIP_BUG)
        setMargin(self.report_button, 100, 5, 5)
        self.grid.attach_next_to(self.report_button, self.faq_button, Gtk.PositionType.RIGHT, 1, 1)

        self.donate_button = Gtk.LinkButton.new_with_label(const.PAYPAL_LINK)
        self.donate_button.set_tooltip_text(const.TOOLTIP_DONATE)
        setMargin(self.donate_button, 100, 5, 5)
        self.grid.attach_next_to(self.donate_button, self.report_button, Gtk.PositionType.RIGHT, 1, 1)

        self.github_button = Gtk.LinkButton.new_with_label(const.GITHUB_LINK)
        self.github_button.set_tooltip_text(const.TOOLTIP_GITHUB)
        setMargin(self.github_button, 100, 5, 5)
        self.grid.attach_next_to(self.github_button, self.donate_button, Gtk.PositionType.RIGHT, 1, 1)

        self.contact_button = Gtk.LinkButton.new_with_label(const.EMAIL_LINK)
        self.contact_button.set_tooltip_text(const.TOOLTIP_CONTACT)
        setMargin(self.contact_button, 100, 5, 5)
        self.grid.attach_next_to(self.contact_button, self.github_button, Gtk.PositionType.RIGHT, 1, 1)
        
        self.refresh_icons()

    def refresh_icons(self):
        self.log_button.set_child(Gtk.Picture.new_for_pixbuf(fetchImageFromUrl(const.LOG_LOGO_PNG, const.ICON_WIDTH, const.ICON_HEIGHT2, True)))
        self.license_button.set_child(Gtk.Picture.new_for_pixbuf(fetchImageFromUrl(const.LICENSE_LOGO_PNG, const.ICON_WIDTH, const.ICON_HEIGHT2, True)))
        self.faq_button.set_child(Gtk.Picture.new_for_pixbuf(fetchImageFromUrl(const.FAQ_LOGO_PNG, const.ICON_WIDTH, const.ICON_HEIGHT2, True)))
        self.report_button.set_child(Gtk.Picture.new_for_pixbuf(fetchImageFromUrl(const.BUG_LOGO_PNG, const.ICON_WIDTH, const.ICON_HEIGHT2, True)))
        self.donate_button.set_child(Gtk.Picture.new_for_pixbuf(fetchImageFromUrl(const.DONATE_LOGO_PNG, const.ICON_WIDTH, const.ICON_HEIGHT2, True)))
        self.github_button.set_child(Gtk.Picture.new_for_pixbuf(fetchImageFromUrl(const.GITHUB_LOGO_PNG, const.ICON_WIDTH, const.ICON_HEIGHT2, True)))
        self.contact_button.set_child(Gtk.Picture.new_for_pixbuf(fetchImageFromUrl(const.CONTACT_LOGO_PNG, const.ICON_WIDTH, const.ICON_HEIGHT2, True)))

def about_page(tab3):
    page = AboutPage()
    tab3.append(page)
    return page


# Copyright (C) 2017-2026 Arun Sivaraman <arunsivaramanneo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

mkdir_output_command = "mkdir /tmp/gpu-viewer"
rmdir_output_command = "rm /tmp/gpu-viewer -r"
vulkaninfo_output_command = "vulkaninfo --show-formats"
vdpauinfo_output_command = "vdpauinfo"
clinfo_output_command = "clinfo -a | awk '/Number of platforms/{flag=1;print}/NULL.*/{flag=0}flag' "
opengl_output_command = "glxinfo -s"
fetch_vulkaninfo_ouput_command = "cat /tmp/gpu-viewer/vulkaninfo.txt |"

fetch_screen_resolution_command = "xdpyinfo | awk '/dimensions/{print $2}'"

unset_LC_ALL_conmand = "unset LC_ALL"

#------------------------- Command Commands --------------------------------

remove_rhs_Command = "awk '{gsub(/[=,:].*/,\"\")}1' | awk '/./' "
remove_lhs_command = "awk -F'[=,:]' '{if (NF>1) {v=$0; sub(/.*[=,:] */, \"\", v); print v} else print \"\"}' "



# ---------------------------Vulkan Tab - Commands and Filenames -------------------------------------

gpu_viewer_folder_path = "/tmp/gpu-viewer"

vulkaninfo_output_file = "/tmp/gpu-viewer/vulkaninfo.txt"

# --------------------------- Vulkan Deviceinfo - Commands and Filenames -------------------------

vulkan_device_info_file = "/tmp/gpu-viewer/VKDDeviceinfotemp.txt"
vulkan_device_info_lhs_file = "/tmp/gpu-viewer/VKDDeviceinfotempLHS.txt"
vulkan_device_info_rhs_file = "/tmp/gpu-viewer/VKDDeviceinfotempRHS.txt"

vulkan_summary_command = "vulkaninfo --summary"

fetch_device_name_command = " grep deviceName | grep -o  =.* | grep -o ' .*'"
fetch_device_id_command = " grep deviceID | grep -o  =.* | grep -o ' .*'"
fetch_vendor_id_command = " grep vendorID | grep -o  =.* | grep -o ' .*'"


#---------------------------- Vulkan Properties and Commands -----------------------------------------------

vulkan_device_properties_file = "/tmp/gpu-viewer/VKDDevicePropertiesTemp.txt"
vulkan_device_filter_properties_file = "/tmp/gpu-viewer/VKDDevicePropertiesFilterTemp.txt"
vulkan_device_filter_properties_lhs_file = "/tmp/gpu-viewer/VKDDevicePropertiesFilterLHSTemp.txt"

 #------------------- Vulkan Features Files and Commands ------------------------------------------------

vulkan_device_features_file = "/tmp/gpu-viewer/VKDDeviceFeaturesTemp.txt"
vulkan_device_features_lhs_file = "/tmp/gpu-viewer/VKDDeviceFeaturesLHS.txt"
vulkan_device_features_select_file = "/tmp/gpu-viewer/VKDDeviceFeaturesRHS.txt"

#---------------------Vulkan Extensions File and Command Name ------------------------------------

vulkan_device_extensions_file = "/tmp/gpu-viewer/VKDDeviceExtensionTemp.txt"
vulkan_device_extension_lhs_file = "/tmp/gpu-viewer/VKDDeviceExtensionLHS.txt"
vulkan_device_extension_rhs_file = "/tmp/gpu-viewer/VKDDeviceExtensionRHS.txt"

#........................ Vulkan Limits commands and File Names -------------------------------------

vulkan_device_limits_file = "/tmp/gpu-viewer/VKDDeviceLimitsTemp.txt"
vulkan_device_limits_lhs_file = "/tmp/gpu-viewer/VKDDeviceLimitsLHS.txt"
vulkan_device_limits_rhs_file = "/tmp/gpu-viewer/VKDDeviceLimitsRHS.txt"

#.........................Vulkan Formats Commands and File Names ----------------------------------

vulkan_device_formats_file = "/tmp/gpu-viewer/VKDDeviceFormatsTemp.txt"
vulkan_device_formats_types_file = "/tmp/gpu-viewer/VKDDeviceFormatsTypesTemp.txt"
vulkan_device_format_types_count_file = "/tmp/gpu-viewer/VKDDeviceFormatTypesCountTemp.txt"
vulkan_device_format_types_linear_count_file = "/tmp/gpu-viewer/VKDDeviceFormatTypesLinearCountTemp.txt"
vulkan_device_format_types_optimal_count_file = "/tmp/gpu-viewer/VKDDeviceFormatTypesOptimalCountTemp.txt"
vulkan_device_format_types_buffer_count_file = "/tmp/gpu-viewer/VKDDeviceFormatTypesBufferCountTemp.txt"
vulkan_device_format_types_linear_file = "/tmp/gpu-viewer/VKDDeviceFormatTypesLinearTypesTemp.txt"
vulkan_device_format_types_optimal_file = "/tmp/gpu-viewer/VKDDeviceFormatTypesOptimalTypesTemp.txt"
vulkan_device_format_types_buffer_file = "/tmp/gpu-viewer/VKDDeviceFormatTypesBufferTypesTemp.txt"

#----------------------Vulkan Device Memory Types Commands and File Names ----------------------------------

vulkan_device_memory_types_file = "/tmp/gpu-viewer/VKDDeviceMemoryTypesTemp.txt"
vulkan_device_memory_types_lhs_file = "/tmp/gpu-viewer/VKDDeviceMemoryTypesLHS.txt"
vulkan_device_memory_types_rhs_file = "/tmp/gpu-viewer/VKDDeviceMemoryTypesRHS.txt"
vulkan_device_memory_types_property_flags_file = "/tmp/gpu-viewer/VKDDeviceMemoryTypesPropertyFlagsTemp.txt"

#-----------------------Vulkan Device Memory Heaps Commands and File Names --------------------------------

vulkan_device_memory_heaps_file = "/tmp/gpu-viewer/VKDDeviceMemoryHeapsTemp.txt"
vulkan_device_memory_heaps_lhs_file = "/tmp/gpu-viewer/VKDDeviceMemoryHeapsLHSTemp.txt"
vulkan_device_memory_heaps_rhs_file = "/tmp/gpu-viewer/VKDDeviceMemoryHeapsRHSTemp.txt"


#---------------------- Vulkan Device Queues  Filenames --------------------------------------------------------------------------

vulkan_device_queues_file = "/tmp/gpu-viewer/VKDDeviceQueuesTemp.txt"
vulkan_device_queues_lhs_file = "/tmp/gpu-viewer/VKDDeviceQueuesLHSTemp.txt"
vulkan_device_queues_rhs_file = "/tmp/gpu-viewer/VKDDeviceQueuesRHSTemp.txt"
vulkan_device_queue_counts_file = "/tmp/gpu-viewer/VKDDeviceQueueCountTemp.txtS"

#------------------- vulkan Device instances Filenames -----------------------------------------------------------------------------

vulkan_device_instances_file = "/tmp/gpu-viewer/VKDDeviceInstancesTemp.txt"
vulkan_device_instances_lhs_file = "/tmp/gpu-viewer/VKDDeviceInstancesLHSTemp.txt"
vulkan_device_instances_rhs_file = "/tmp/gpu-viewer/VKDDeviceInstancesRHSTemp.txt"

#--------------------vulkan Device layers Filenames -----------------------------------------------------------------------------------

vulkan_device_layers_file = "/tmp/gpu-viewer/VKDDeviceLayersTemp.txt"
vulkan_device_layers_name_file = "/tmp/gpu-viewer/VKDDeviceLayerNames.txt"


#-------------------vulkan Device Surface Filenames -------------------------------------------------------------------------------

vulkan_device_surface_file = "/tmp/gpu-viewer/VKDDeviceSurfaceTemp.txt"

#-------------------vulkan Device Groups Filenames -------------------------------------------------------------------------------

vulkan_device_groups_file = "/tmp/gpu-viewer/VKDDeviceGroupsTemp.txt"


#-------------------------- OpenGL Files and Commands --------------------------------------------------------

opengl_outpuf_file = "/tmp/gpu-viewer/glxinfo.txt"
opengl_device_info_file = "/tmp/gpu-viewer/opengl_information.txt"
opengl_info_lhs_file = "/tmp/gpu-viewer/opengl_information_lhs.txt"
opengl_info_rhs_file = "/tmp/gpu-viewer/opengl_information_rhs.txt"
opengl_vendor_gl_extension_file = "/tmp/gpu-viewer/opengl_vendor_gl_extension.txt"
opengl_vendor_es_extension_file = "/tmp/gpu-viewer/opengl_vendor_es_extension.txt"
egl_vendor_extension_file = "/tmp/gpu-viewer/egl_vendor_extension.txt"


fetch_opengl_vendor_extensions_command = "cat %s | awk '/OpenGL extensions/{flag=1;next}/OpenGL ES profile/{flag=0} flag' | grep GL_ | sort" %(opengl_outpuf_file)
fetch_openglx_vendor_extensions_command ="cat %s  | awk '/client glx extensions/{flag=1; next}/GLX version/{flag=0} flag' | grep GLX_ | sort" %(opengl_outpuf_file)

fetch_opengl_es_vendor_extensions_command = "cat %s  | awk '/OpenGL ES profile/{flag=1;next}/80 GLX Visuals/{flag=0} flag' | grep GL_ | sort" %(opengl_outpuf_file)

fetch_egl_vendor_extension_command = "es2_info | awk '/EGL_EXTENSIONS.*/{flag=1;next}/EGL_CLIENT.*/{flag=0}flag'| awk '{n=split($0,a,/,/);{for (i=1;i<=n;i++) print a[i]}}' | grep -o EGL.* "

#----------------------- OpenGL Limits ------------------------------------------------------------------------------------------------------------------

opengl_core_limits_lhs_file = "/tmp/gpu-viewer/opengl_core_limits_lhs.txt"
opengl_core_limits_file = "/tmp/gpu-viewer/opengl_core_limits.txt"

fetch_opengl_core_limits_command = "glxinfo -l | awk '/OpenGL core profile limits:/{flag=1}/GLX Visuals.*/{flag=0} flag' | awk '/OpenGL core profile limits:/{flag=1;next}/OpenGL version string.*/{flag=0} flag' | awk '/./' "
fetch_opengl_core_limits_lhs_command = "cat %s | awk '{gsub(/=.*/,\"\")}1' " %(opengl_core_limits_file)

opengl_compat_limits_file = "/tmp/gpu-viewer/opengl_compat_limits.txt"

opengl_compat_limits_lhs_file = "/tmp/gpu-viewer/opengl_compat_limits_lhs.txt"

fetch_opengl_compat_limits_command = "glxinfo -l | awk '/OpenGL limits:/{flag=1}/GLX Visuals.*/{flag=0} flag' | awk '/OpenGL limits:/{flag=1;next}/OpenGL ES profile/{flag=0} flag' | awk '/./' "

fetch_opengl_compat_limits_lhs_command = "cat %s | awk '{gsub(/=.*/,\"\")}1' " %(opengl_compat_limits_file)

select_opengl_limits_file = "/tmp/gpu-viewer/selectOpenglLimits.txt"


#-------------------------------------------- Clinfo filenames and Commands -------------------------------------------------------------------

opencl_output_file = "/tmp/gpu-viewer/clinfo.txt"

opencl_plaform_and_device_names_file = "/tmp/gpu-viewer/PlatnDev.txt"


fetch_platform_and_device_names_command = "clinfo -l"

fetch_platform_names_command = "cat %s | grep Platform | grep -o :.* | grep -o ' .*' " %(opencl_plaform_and_device_names_file)

opencl_platform_details_file = "/tmp/gpu-viewer/oclPlatformDetails.txt"

fetch_platform_details_lhs_command = "cat %s | grep -o Platform.* | awk '{gsub(/  .*/,'True');print}' " %(opencl_platform_details_file)

fetch_platform_details_rhs_command = "cat %s | grep -o Platform.* | awk '{gsub(/Platform.*  /,'True');print}'" %(opencl_platform_details_file)

opencl_device_details_file = "/tmp/gpu-viewer/oclDeviceDetails.txt"

fetch_device_details_lhs_command = "cat %s | awk '{gsub(/     .*/,'True');print}'" %(opencl_device_details_file)
fetch_device_details_rhs_command = "cat %s | awk '{print substr($0,51,10000);}'" %(opencl_device_details_file)

opencl_device_memory_and_image_file = "/tmp/gpu-viewer/oclDeviceMemoryImageDetails.txt"

fetch_device_memory_and_image_details_lhs_command = "cat %s | awk '{gsub(/     .*/,'True');print}'" %(opencl_device_memory_and_image_file)
fetch_device_memory_and_image_details_rhs_command = "cat %s | awk '{print substr($0,51,500);}'"  %(opencl_device_memory_and_image_file)

opencl_device_vector_file = "/tmp/gpu-viewer/oclDeviceVectorDetails.txt"

fetch_device_vector_details_lhs_command = "cat %s | awk '{gsub(/     .*/,'True');print}'" %(opencl_device_vector_file)
fetch_device_vector_details_rhs_command = "cat %s | awk '{gsub(/^ .*        /,'True');print}'" %(opencl_device_vector_file)

opencl_device_queue_execution_details_file = "/tmp/gpu-viewer/oclDeviceQueueExecutionDetails.txt"

fetch_device_queue_execution_details_lhs_command = "cat %s | awk '{gsub(/     .*/,'True');print}'" %(opencl_device_queue_execution_details_file)
fetch_device_queue_execution_details_rhs_command = "cat %s | awk '{print substr($0,51,500);}'" %(opencl_device_queue_execution_details_file)

# ------------------------------------------- VDPAUINFO filenames and Commands ------------------------------------------------------------

vdpauinfo_output_file = "/tmp/gpu-viewer/vdpauinfo.txt"

#---------------------------------------------- Themes Folder ----------------------------------------------------------------------------------------------------

fetch_theme_preference = "gsettings get org.gnome.desktop.interface color-scheme"
Orchis_gtk_theme_folder = "/usr/share/themes/Orchis"
Materia_gtk_theme_folder = "/usr/share/themes/Materia"
Roboto_font_folder = "/usr/share/fonts/truetype/roboto"



# Copyright (C) 2017-2026 Arun Sivaraman <arunsivaramanneo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import os.path as Path
import gi
gi.require_version("Adw", "1")
from gi.repository import Gtk, Adw, GObject,Gdk, Gio


def update_theme_constants(prefer_dark):
    global AMDRADEON_LOGO_PNG, Gnome_logo, LLVM_logo, LLVM_LOGO_SVG, AMDRYZEN_LOGO_PNG
    global AMD_LOGO_PNG, GITHUB_LOGO_PNG, CONTACT_LOGO_PNG, BUG_LOGO_PNG, cosmic_logo
    global BGCOLOR1, BGCOLOR2, BGCOLOR3, COLOR1, COLOR2, COLOR3, OPEN_CL_PNG, Ryzen_logo, AMD_logo, CACHYOS_LOGO_PNG
    
    if prefer_dark:
        AMDRADEON_LOGO_PNG = "../Images/AMD_Radeon_White.png"
        Gnome_logo = "../Images/Gnome_logo.png"
        LLVM_logo = "../Images/LLVM_Logo.png"
        LLVM_LOGO_SVG = "../Images/LLVM_Logo.png"
        AMDRYZEN_LOGO_PNG = "../Images/amd-ryzen.png"
        AMD_LOGO_PNG = "../Images/AMD_D.png"
        AMD_logo = "../Images/AMD_D.png"
        GITHUB_LOGO_PNG = "../Images/github.png"
        CONTACT_LOGO_PNG = "../Images/Contact.png"
        BUG_LOGO_PNG = "../Images/bug.png"
        cosmic_logo = "../Images/cosmic.png"
        OPEN_CL_PNG = "../Images/OpenCL.svg"
        Ryzen_logo = "../Images/Ryzen.svg"
        BGCOLOR1 = "#353839"
        BGCOLOR2 = "#2D2E36"
        BGCOLOR3 = "#171717"
        COLOR1 = "GREEN"
        COLOR2 = "RED"
        COLOR3 = "#FFFFFF"
    else:
        AMDRADEON_LOGO_PNG = "../Images/AMDRadeon.png"
        Gnome_logo = "../Images/Gnome_Dark.png"
        LLVM_logo = "../Images/LLVM-Logo.svg"
        LLVM_LOGO_SVG = "../Images/LLVM-Logo.svg"
        AMDRYZEN_LOGO_PNG = "../Images/AMD_Ryzen.svg"
        AMD_LOGO_PNG = "../Images/AMD_logo.png"
        AMD_logo = "../Images/AMD_logo.png"
        GITHUB_LOGO_PNG = "../Images/github.png"
        CONTACT_LOGO_PNG = "../Images/Contact.png"
        BUG_LOGO_PNG = "../Images/bug.png"
        cosmic_logo = "../Images/Cosmic_white.svg"
        OPEN_CL_PNG = "../Images/OpenCL_Dark.png"
        Ryzen_logo = "../Images/Ryzen_Dark.svg"
        BGCOLOR1 = "#e9edf6"
        BGCOLOR2 = "#eeeeee"
        BGCOLOR3 = "#ccc"
        COLOR1 = "GREEN"
        COLOR2 = "RED"
        COLOR3 = "#333333"

update_theme_constants(False)







    

GTK_CSS = "gtk.css"

GTK_DARK_CSS = "gtk_dark.css"

ABOUT_US_PNG = "../Images/about-us.png"

VULKAN_PNG = "../Images/Vulkan.png"

OPEN_GL_PNG = "../Images/OpenGL.png"

GLX_PNG = "../Images/glx.png"

OPEN_GL_ES_PNG = "../Images/OpenGL_ES.png"

# OPEN_CL_PNG is dynamic

EGL_PNG  = "../Images/Egl_logo.png"

VDPAU_CL_PNG = "../Images/vdpauinfo.png"

NVIDIA_GTX_LOGO_PNG = "../Images/nvidia-logo.png"

NVIDIA_RTX_LOGO_PNG = "../Images/nvidia-RTX-logo.png"

NVIDIA_CUDA_LOGO_PNG = "../Images/Cuda.png"

GEFORCE_PNG = "../Images/GeForce.png"

INTEL_LOGO_PNG = "../Images/intel-logo.png"

INTEL_ARC_LOGO_PNG = "../Images/Intel_Arc_logo.png"

CACHYOS_LOGO_PNG = "../Images/CachyOS_Logo.png"

#AMD_LOGO_PNG = "../Images/AMD.png"


AMD_RADEON_VEGA_LOGO_PNG = "../Images/radeon-vega.png"

#AMDRADEON_LOGO_PNG = "../Images/AMD_Radeon_White.png"


AMD_Ryzen_RADEON_LOGO_PNG = "../Images/AMD_Ryzen_Radeon.png"

AMD_RADEON_Pro_LOGO_PNG = "../Images/AMD_Radeon_Pro.png"



#AMDRYZEN_LOGO_PNG = "../Images/AMD_Ryzen.png"

POCL_LOGO_PNG = "../Images/pocl.png"

CUDA_PNG = "../Images/Cuda.png"

#LLVM_LOGO_SVG = "../Images/LLVM_Logo.png"

MESA_LOGO_PNG = "../Images/mesa-logo.png"

INTEL_BANNER = "../Images/Intel_Banner.png"

NVIDIA_BANNER = "../Images/nvidia-banner.jpg"

TWITTER_LOGO_PNG = "../Images/twitter-icon.png"

VULKAN_VIDEO_PNG = "../Images/Vulkan-Video.png"

DONATE_LOGO_PNG = "../Images/Donate.png"


FAQ_LOGO_PNG = "../Images/faq-icon.png"

LICENSE_LOGO_PNG = "../Images/GPL3.png"


LOG_LOGO_PNG = "../Images/Changelog.png"

APP_LOGO_PNG = "../Images/GPU_Viewer.png"

NVIDIA_LOGO_PNG = "../Images/Nvidia_logo.png"

DISCORD_LOGO_PNG = "../Images/Discord.png"

TOOLTIP_LICENSE = "View License"

TOOLTIP_CHANGE_LOG = "View Change Log"

LICENSE_HTML_LINK = "https://www.gnu.org/licenses/gpl-3.0-standalone.html"

CHANGE_LOG_LINK = "https://github.com/arunsivaramanneo/GPU-Viewer/releases"

TOOLTIP_FAQ = "View Frequently Asked Questions"

FAQ_LINK = "https://github.com/arunsivaramanneo/GPU-Viewer/wiki/FAQ----GPU-Viewer"

TOOLTIP_BUG = "Click here to report a bug"

ISSUE_LINK = "https://github.com/arunsivaramanneo/GPU-Viewer/issues"

EMAIL_LINK = "mailto:arunsivaramanneo@gmail.com?Subject=GPU Viewer - <Enter Description here>"

TOOLTIP_DONATE = "Donate PayPal "

PAYPAL_LINK = "https://www.paypal.com/donate/?hosted_button_id=7M3PMM78FBR4Q"

TOOLTIP_TWITTER = "Follow me on Twitter"

TWITTER_LINK = "https://twitter.com/arunsivaraman"

TOOLTIP_GITHUB = "Arun Sivaraman's Github"

GITHUB_LINK = "https://github.com/arunsivaramanneo/GPU-Viewer"

DISCORD_LINK = "https://discord.gg/S7YbaWWM"

DUMMY_PIXBUF = "../Images/8e8e8e.png"

TRANSPARENT_PIXBUF = "../Images/2c2c2c.png"


# ----------------- Distro logos --------------------------------------

Ubuntu_logo = "../Images/Ubuntu.png"
Mint_logo = "../Images/linux_mint.png"
Manjaro_logo = "../Images/Manjaro-log.png"
Open_Suse_logo = "../Images/open_suse.png"
fedora_logo = "../Images/Fedora_icon.png"
Pop_os_logo = "../Images/pop_os.png"
Flatpak_logo = "../Images/Flatpak.png"
Elementary_logo = "../Images/elementaryOS.png"
Debian_logo = "../Images/debian.png"
Arch_logo = "../Images/arch.png"
Solus_logo = "../Images/Solus.svg"
Xubuntu_logo = "../Images/xubuntu.png"
Lubuntu_logo = "../Images/Lubuntu_logo.svg"
Kubuntu_logo = "../Images/Kubuntu_logo.png"
MX_linux_logo = "../Images/MX_Linux_logo.svg"
Zorin_os_logo = "../Images/Zorin_Logo.svg"
Linux_Lite_logo = "../Images/Linux-Lite-Logo.png"
OpenMandriva_logo = "../Images/Mandriva.png"
Ubuntu_Budgie_logo = "../Images/Ubuntu_Budgie.png"
Ubuntu_Studio_logo = "../Images/Ubuntu_Studio.png"
RebornOS_logo = "../Images/RebornOS_Icon.png"
NixOS_logo = "../Images/NixOS.png"
Rhino_Linux_logo = "../Images/Rhino_Linux.png"
Steam_OS_logo = "../Images/Steam_icon_logo.png"
Nobara_OS_logo = "../Images/nobara.png"
Anduinos_logo = "../Images/anduinos_logo.svg"

# These logos are updated in update_theme_constants
# Nvidia_logo, Intel_logo, Mesa_logo, Ryzen_logo are handled dynamically
Mesa_logo = "../Images/mesa.png" # Mesa doesn't have a clear variant yet, keeping default
Intel_logo = "../Images/Intel_logo.png" # Keeping default for now
Nvidia_logo = "../Images/nvidia_logo.png" # Keeping default for now


#------------------- Desktop logo--------------------------------------

#Gnome_logo = "../Images/Gnome_logo.png"
Unity_logo = "../Images/Unity_logo.png"
Cinnamon_logo = "../Images/Cinnamon-logo.png"
Kde_logo = "../Images/kde-logo.png"
Budgie_logo = "../Images/Budgie.png"
Mate_logo = "../Images/Mate-logo.svg"
XFCE_logo = "../Images/xfce-logo.svg"
Fluxbox_logo = "../Images/Fluxbox-logo.png"
Sway_logo = "../Images/Sway_Tree.svg"
Wayland_logo = "../Images/Wayland_Logo.svg"
X11_logo = "../Images/X11_Logo.png"

#Gnome_logo = "../Images/Gnome_Dark.png"

TOOLTIP_CONTACT = "Contact us"

FRAMEBUFFERLIST = ["vid", "vdep", "vt", "xsp", "bfsz", "lvl", "rt", "db", "st", "rsz", "gsz", "bsz", "asz", "flt",
                   "srgb", "aux", "depth", "stcl",
                   "acr", "acg", "acb", "aca", "msnum", "msbufs", "Swap", "caveats"]

ICON_WIDTH = 350

ICON_HEIGHT = 50

ICON_HEIGHT2 = 50

WIDTH_RATIO = 0.86

WIDTH_RATIO2 = 0.70

HEIGHT_RATIO1 = 0.90    

HEIGHT_RATIO2 = 0.65

MWIDTH = 350


# ------------------------ theme Menu -----------------------------------------

MENU_XML="""
<?xml version="1.0" encoding="UTF-8"?>
<interface>
  <menu id="app-menu">
    <section>
      <item>
        <attribute name="action">win.about</attribute>
        <attribute name="label" translatable="yes">_Light theme</attribute>
      </item>
      <item>
        <attribute name="action">win.quit</attribute>
        <attribute name="label" translatable="yes">_Dark theme</attribute>
    </item>
    </section>
  </menu>
</interface>"""



# Copyright (C) 2017-2026 Arun Sivaraman <arunsivaramanneo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import gi
import  const
import subprocess
import re
import Filenames

gi.require_version("Gtk", "4.0")
gi.require_version("Adw", "1")
from gi.repository import Gtk,Gio,Adw,GdkPixbuf

from Common import setup_expander, createSubTab, create_scrollbar, bind_expander,setMargin, setup,bind1, createMainFile, fetchContentsFromCommand,ExpandDataObject,add_tree_node,getGpuImage

platformDetailsHeader = ["Platform Information ", "Details "]
deviceDetailsHeader = ["Device Information ", "Details "]
deviceMemoryImageHeader = ["Device Information ", "Details "]


class ClinfoParser:
    def __init__(self):
        self.platforms = []
        self.fetch_data()

    def fetch_data(self):
        # Fix #3: Re-use the clinfo output file already written by
        # isOpenclSupported() in gpu_viewer.py — avoids a redundant
        # clinfo subprocess on every startup.
        try:
            with open(Filenames.opencl_output_file, "r") as f:
                output = f.read()
            if not output.strip():
                raise ValueError("clinfo output file is empty")
            self.platforms = self.parse(output)
        except Exception as e:
            print(f"Error reading clinfo output: {e}")
            # Fallback: run clinfo directly if the file is missing/empty
            try:
                output = subprocess.check_output(["clinfo"], universal_newlines=True)
                self.platforms = self.parse(output)
            except Exception as e2:
                print(f"Error running clinfo fallback: {e2}")
                self.platforms = []

    def parse(self, output):
        platforms = []
        platform_map = {}
        current_platform = None
        current_device = None
        mode = "START"
        
        lines = output.splitlines()
        for line in lines:
            if not line.strip(): continue
            indent = len(line) - len(line.lstrip())
            content = line.strip()
            
            if indent == 0:
                if "Number of platforms" in content:
                    mode = "PLATFORM"
                elif "Number of devices" in content:
                    mode = "DEVICE"
                    current_device = None
                elif "ICD loader properties" in content or "NULL platform behavior" in content:
                    mode = "STOP"
                continue

            if mode == "STOP" or mode == "START":
                continue

            # Split key and value (usually separated by multiple spaces)
            if "  " in content:
                parts = re.split(r'\s{2,}', content, maxsplit=1)
                key = parts[0].strip()
                value = parts[1].strip()
            elif " 0x" in content:
                parts = content.split(" 0x", 1)
                key = parts[0].strip()
                value = "0x" + parts[1].strip()
            else:
                key = content
                value = ""

            if indent == 2:
                if key == "Platform Name":
                    if value not in platform_map:
                        current_platform = {"name": value, "properties": [], "devices": []}
                        platforms.append(current_platform)
                        platform_map[value] = current_platform
                    else:
                        current_platform = platform_map[value]
                    continue
                
                if mode == "PLATFORM" and current_platform:
                    # Avoid duplicate properties if clinfo repeats platform block
                    if not any(p[0] == key for p in current_platform["properties"]):
                        current_platform["properties"].append((key, value, []))
                elif mode == "DEVICE" and current_platform:
                    if key == "Device Name":
                        current_device = {"name": value, "properties": []}
                        current_platform["devices"].append(current_device)
                    elif current_device:
                        current_device["properties"].append((key, value, []))
            elif indent > 2:
                # Sub-property of the last property
                if current_device and current_device["properties"]:
                    current_device["properties"][-1][2].append((key, value))
                elif current_platform and current_platform["properties"]:
                    current_platform["properties"][-1][2].append((key, value))
                    
        return platforms

def openCL(self, tab):
    gpu_index_map = []
    self.opencl_gpu_index_map = gpu_index_map
    parser = ClinfoParser()
    oclPlatforms = parser.platforms
    self.opencl_platforms = oclPlatforms
    platformDetails_Store = Gio.ListStore.new(ExpandDataObject)
    PlatformExtensionDetails_Store = Gio.ListStore.new(ExpandDataObject)
    DeviceDetails_Store = Gio.ListStore.new(ExpandDataObject)
    DeviceExtensionDetails_Store = Gio.ListStore.new(ExpandDataObject)
    DeviceMemoryImage_store = Gio.ListStore.new(ExpandDataObject)
    DeviceVector_store = Gio.ListStore.new(ExpandDataObject)
    DeviceQueueExecution_store = Gio.ListStore.new(ExpandDataObject)


    def getPlatformNames():
        return [p["name"] for p in oclPlatforms]

    def selectDevice(dropdown,dummy):
        selected =dropdown.props.selected_item
        DeviceDetails_Store.remove_all()
        DeviceExtensionDetails_Store.remove_all()
        PlatformExtensionDetails_Store.remove_all()
        DeviceMemoryImage_store.remove_all()
        DeviceVector_store.remove_all()
        DeviceQueueExecution_store.remove_all()
        value = 0
        if selected is not None:
            value = dropdown.props.selected
            if value == Gtk.INVALID_LIST_POSITION:
                value = 0
            real_idx = gpu_index_map[value] if len(gpu_index_map) > value else value
            getDeviceDetails(real_idx)
            refresh_extensions()
            getDeviceMemoryImageDetails(real_idx)
            getDeviceVectorDetails(real_idx)
            getDeviceQueueExecutionCapabilities(real_idx)
        
            gpu_image = getGpuImage(selected.get_string())
            image_renderer.set_pixbuf(gpu_image)


    def getDeviceNames(value):
        if value >= len(oclPlatforms): return
        
        platform = oclPlatforms[value]
        Devices_list = Gtk.StringList()
        Devices_dropdown.set_model(Devices_list)
        
        oclDeviceNames = [d["name"] for d in platform["devices"]]

        numberOfDevicesEntry.set_text(str(len(oclDeviceNames)))
        numberOfDevicesEntry.set_editable(False)

        gpu_index_map.clear()
        unique_names = []
        for i, name in enumerate(oclDeviceNames):
            # Keep unique names for dropdown but map back to original index
            if name not in unique_names:
                unique_names.append(name)
                gpu_index_map.append(i)
                Devices_list.append(name)

    def getPlatfromDetails(value):
        if value >= len(oclPlatforms): return
        
        platform = oclPlatforms[value]
        all_props = platform["properties"]
        populate_store(platformDetails_Store, all_props)

    def get_device_categories(platform_idx, device_idx):
        if platform_idx >= len(oclPlatforms): return None
        platform = oclPlatforms[platform_idx]
        if device_idx >= len(platform["devices"]): return None
        device = platform["devices"][device_idx]
        
        categories = {
            "Device Information": [],
            "Device Extensions": [],
            "Device Extensions with Version": [],
            "Device Memory & Image Information": [],
            "Queue & Execution Capabilities": [],
            "Device Vector Information": []
        }
        
        current_cat = "Device Information"
        for key, value, children in device["properties"]:
            # Skip redundant platform info in device section
            if key == "Platform Name" or key == "Platform Vendor":
                continue

            if "Device Extensions with Version" in key:
                categories["Device Extensions with Version"].append((key, value, children))
                continue
            if "Device Extensions" in key:
                categories["Device Extensions"].append((key, value, children))
                continue
            
            if "Preferred / native vector sizes" in key:
                current_cat = "Device Vector Information"
            elif "Address bits" in key or "Max alignment" in key:
                current_cat = "Device Memory & Image Information"
            elif "Queue properties" in key or "Execution capabilities" in key or "Device enqueue" in key:
                current_cat = "Queue & Execution Capabilities"
                
            categories[current_cat].append((key, value, children))
        return categories
    self.opencl_get_device_categories = get_device_categories

    def populate_store(store, props, skip_top=False, add_children=True):
        store.remove_all()
        for key, value, children in props:
            if ("Extensions" in key or "Built-in kernels" in key or "Atomic" in key or "features" in key or "all versions" in key or "USM" in key) and "n/a" not in value.lower():
                # Handle long lists as expandable
                all_children = []
                if value:
                    if "with Version" in key or "with version" in key or "features" in key or "all versions" in key or "IL" in key:
                        if "  " in value:
                            parts = re.split(r'\s{2,}', value, maxsplit=1)
                            all_children.append(ExpandDataObject(parts[0].strip(), parts[1].strip()))
                        elif " 0x" in value:
                            parts = value.split(" 0x", 1)
                            all_children.append(ExpandDataObject(parts[0].strip(), "0x" + parts[1].strip()))
                        elif " (" in value:
                            parts = value.split(" (", 1)
                            all_children.append(ExpandDataObject(parts[0].strip(), "(" + parts[1].strip()))
                        else:
                            all_children.append(ExpandDataObject(value.strip(), ""))
                    elif "Extensions" in key:
                        for ext in value.split():
                            all_children.append(ExpandDataObject(ext, ""))
                    elif "Atomic" in key or "USM" in key:
                        for item in value.split(','):
                            all_children.append(ExpandDataObject(item.strip(), ""))
                    else:
                        for item in value.split(';'):
                            all_children.append(ExpandDataObject(item.strip(), ""))
                
                # Add versioned children if any
                for sub_key, sub_val in children:
                    all_children.append(ExpandDataObject(sub_key, sub_val))
                
                if skip_top:
                    for child in all_children:
                        store.append(child)
                else:
                    toprow = ExpandDataObject(key, str(len(all_children)))
                    if add_children:
                        toprow.children = all_children
                    store.append(toprow)
            else:
                toprow = ExpandDataObject(key, value)
                for sub_key, sub_val in children:
                    toprow.children.append(ExpandDataObject(sub_key, sub_val))
                store.append(toprow)

    def getDeviceDetails(value):
        platform_idx = platform_dropdown.props.selected
        cats = get_device_categories(platform_idx, value)
        if cats:
            populate_store(DeviceDetails_Store, cats["Device Information"])

    def getDeviceMemoryImageDetails(value):
        platform_idx = platform_dropdown.props.selected
        cats = get_device_categories(platform_idx, value)
        if cats:
            populate_store(DeviceMemoryImage_store, cats["Device Memory & Image Information"])

    def getDeviceVectorDetails(value):
        platform_idx = platform_dropdown.props.selected
        cats = get_device_categories(platform_idx, value)
        if cats:
            populate_store(DeviceVector_store, cats["Device Vector Information"])

    def getDeviceExtensionDetails(value):
        platform_idx = platform_dropdown.props.selected
        cats = get_device_categories(platform_idx, value)
        if cats:
            populate_store(DeviceExtensionDetails_Store, cats["Device Extensions with Version"], skip_top=True)

    def refresh_extensions():
        if Devices_dropdown.props.selected == Gtk.INVALID_LIST_POSITION:
            return
        real_idx = gpu_index_map[Devices_dropdown.props.selected] if len(gpu_index_map) > Devices_dropdown.props.selected else Devices_dropdown.props.selected
        getDeviceExtensionDetails(real_idx)

    def getDeviceQueueExecutionCapabilities(value):
        platform_idx = platform_dropdown.props.selected
        cats = get_device_categories(platform_idx, value)
        if cats:
            populate_store(DeviceQueueExecution_store, cats["Queue & Execution Capabilities"])

    def selectPlatform(dropdown,dummy):
        selected =dropdown.props.selected_item
        value = 0
        if selected is not None:
            value = dropdown.props.selected
        getDeviceNames(value)
        getPlatfromDetails(value)

#        gpu_image = getGpuImage(gpu_list[text])
#        image_renderer.set_pixbuf(gpu_image)

    #    os.system("rm /tmp/gpu-viewer/ocl*.txt")

    split_view_container = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
    split_view_container.set_vexpand(True)
    split_view_container.set_hexpand(True)

    split_view = Adw.NavigationSplitView.new()
    split_view.set_vexpand(True)
    split_view.set_hexpand(True)

    # Create the sidebar content using a Gtk.ListBox for tabs
    sidebar_listbox = Gtk.ListBox.new()
    sidebar_listbox.set_selection_mode(Gtk.SelectionMode.SINGLE)
    sidebar_listbox.set_vexpand(True)
    sidebar_listbox.add_css_class(css_class="boxed-list")
    sidebar_listbox.add_css_class(css_class="sidebar")
    sidebar_listbox.set_show_separators(True)

    tabs = [
        "Platform Information", "Device Information", "Device Extensions", "Device Memory  \n \t    &\nImage Information",
        "Queue Capabilities\n \t\t&\nExecution Capabilities", "Device Vector Information"
    ]


    content_stack = Gtk.Stack.new()
    self.opencl_content_stack = content_stack

    # Create the content pages for each tab and add them to the stack
    for tab_name in tabs:
        row = Gtk.ListBoxRow()
        label = Gtk.Label.new(tab_name)
        # Add padding to the label inside the row
        label.set_margin_top(10)
        label.set_margin_bottom(10)
        row.set_child(label)
        sidebar_listbox.append(row)


        # Create a box for the content of this specific tab
        content_box = Gtk.Box.new(Gtk.Orientation.VERTICAL, 10)

        if "Platform Information" in tab_name:

                platformColumnView = Gtk.ColumnView()
                platformColumnView.props.show_row_separators = True
                platformColumnView.props.show_column_separators = False

                factory_platform = Gtk.SignalListItemFactory()
                factory_platform.connect("setup",setup_expander)
                factory_platform.connect("bind",bind_expander)

                factory_platform_value = Gtk.SignalListItemFactory()
                factory_platform_value.connect("setup",setup)
                factory_platform_value.connect("bind",bind1)

                platformSelection = Gtk.SingleSelection()
                # Store already initialized at top

                platformModel = Gtk.TreeListModel.new(platformDetails_Store,False,True,add_tree_node)
                platformSelection.set_model(platformModel)

                platformColumnView.set_model(platformSelection)

                platformColumnLhs = Gtk.ColumnViewColumn.new("Platform Information",factory_platform)
                platformColumnLhs.set_resizable(True)
                platformColumnRhs = Gtk.ColumnViewColumn.new("Details",factory_platform_value)
                platformColumnRhs.set_expand(True)

                platformColumnView.append_column(platformColumnLhs)
                platformColumnView.append_column(platformColumnRhs)

                platformScrollbar = create_scrollbar(platformColumnView)
                platformScrollbar.set_vexpand(True)

                content_box.append(platformScrollbar)
        elif "Device Information" in tab_name:

                deviceColumnView = Gtk.ColumnView()
                deviceColumnView.props.show_row_separators = True
                deviceColumnView.props.show_column_separators = False

                factory_devices = Gtk.SignalListItemFactory()
                factory_devices.connect("setup",setup_expander)
                factory_devices.connect("bind",bind_expander)

                factory_devices_value = Gtk.SignalListItemFactory()
                factory_devices_value.connect("setup",setup)
                factory_devices_value.connect("bind",bind1)

                deviceSelection = Gtk.SingleSelection()
                # Store already initialized at top

                deviceModel = Gtk.TreeListModel.new(DeviceDetails_Store,False,True,add_tree_node)
                deviceSelection.set_model(deviceModel)

                deviceColumnView.set_model(deviceSelection)

                deviceColumnLhs = Gtk.ColumnViewColumn.new("Device Information",factory_devices)
                deviceColumnLhs.set_resizable(True)
                deviceColumnRhs = Gtk.ColumnViewColumn.new("Details",factory_devices_value)
                deviceColumnRhs.set_expand(True)

                deviceColumnView.append_column(deviceColumnLhs)
                deviceColumnView.append_column(deviceColumnRhs)

                DeviceDetailsScrollbar = create_scrollbar(deviceColumnView)

                content_box.append(DeviceDetailsScrollbar)
        elif tab_name == "Device Extensions":
                deviceExtensionColumnView = Gtk.ColumnView()
                deviceExtensionColumnView.props.show_row_separators = True
                deviceExtensionColumnView.props.show_column_separators = False

                factory_devices_extension = Gtk.SignalListItemFactory()
                factory_devices_extension.connect("setup",setup_expander)
                factory_devices_extension.connect("bind",bind_expander)

                factory_devices_extension_value = Gtk.SignalListItemFactory()
                factory_devices_extension_value.connect("setup",setup)
                factory_devices_extension_value.connect("bind",bind1)

                deviceExtensionSelection = Gtk.SingleSelection()
                # Store already initialized at top

                search_entry = Gtk.SearchEntry()
                search_entry.set_property("placeholder-text", "Search extensions...")
                search_entry.add_css_class(css_class='toolbar')
                setMargin(search_entry, 10, 10, 10)


                def filter_func(item, data):
                    search_text = search_entry.get_text().lower()
                    if not search_text:
                        return True
                    # Check both LHS (data) and RHS (data2)
                    return search_text in item.data.lower() or search_text in item.data2.lower()

                filter_model = Gtk.FilterListModel.new(model=DeviceExtensionDetails_Store)
                custom_filter = Gtk.CustomFilter.new(filter_func, None)
                filter_model.set_filter(custom_filter)

                search_entry.connect("search-changed", lambda w: custom_filter.changed(Gtk.FilterChange.DIFFERENT))

                deviceExtensionModel = Gtk.TreeListModel.new(filter_model, False, True, add_tree_node)
                deviceExtensionSelection.set_model(deviceExtensionModel)

                deviceExtensionColumnView.set_model(deviceExtensionSelection)

                deviceExtensionColumnLhs = Gtk.ColumnViewColumn.new("Device Extensions",factory_devices_extension)
                deviceExtensionColumnLhs.set_resizable(True)
                deviceExtensionColumnRhs = Gtk.ColumnViewColumn.new("Version",factory_devices_extension_value)
                deviceExtensionColumnRhs.set_expand(True)

                deviceExtensionColumnView.append_column(deviceExtensionColumnLhs)
                deviceExtensionColumnView.append_column(deviceExtensionColumnRhs)

                DeviceExtensionDetailsScrollbar = create_scrollbar(deviceExtensionColumnView)

                content_box.append(DeviceExtensionDetailsScrollbar)
                content_box.append(search_entry)
        elif "Device Memory" in tab_name:
                deviceMemoryImageColumnView = Gtk.ColumnView()
                deviceMemoryImageColumnView.props.show_row_separators = True
                deviceMemoryImageColumnView.props.show_column_separators = False

                factory_devices_memory_image = Gtk.SignalListItemFactory()
                factory_devices_memory_image.connect("setup",setup_expander)
                factory_devices_memory_image.connect("bind",bind_expander)

                factory_devices_memory_image_value = Gtk.SignalListItemFactory()
                factory_devices_memory_image_value.connect("setup",setup)
                factory_devices_memory_image_value.connect("bind",bind1)

                deviceMemoryImageSelection = Gtk.SingleSelection()
                # Store already initialized at top

                deviceMemoryImageModel = Gtk.TreeListModel.new(DeviceMemoryImage_store,False,True,add_tree_node)
                deviceMemoryImageSelection.set_model(deviceMemoryImageModel)

                deviceMemoryImageColumnView.set_model(deviceMemoryImageSelection)

                deviceMemoryImageColumnLhs = Gtk.ColumnViewColumn.new("Device Information",factory_devices_memory_image)
                deviceMemoryImageColumnLhs.set_resizable(True)
                deviceMemoryImageColumnRhs = Gtk.ColumnViewColumn.new("Details",factory_devices_memory_image_value)
                deviceMemoryImageColumnRhs.set_expand(True)

                deviceMemoryImageColumnView.append_column(deviceMemoryImageColumnLhs)
                deviceMemoryImageColumnView.append_column(deviceMemoryImageColumnRhs)

                DeviceMemoryImageScrollbar = create_scrollbar(deviceMemoryImageColumnView)

                content_box.append(DeviceMemoryImageScrollbar)
        elif "Queue Capabilities" in tab_name:
                deviceQueueExecutionColumnView = Gtk.ColumnView()
                deviceQueueExecutionColumnView.props.show_row_separators = True
                deviceQueueExecutionColumnView.props.show_column_separators = False

                factory_devices_queue_execution = Gtk.SignalListItemFactory()
                factory_devices_queue_execution.connect("setup",setup_expander)
                factory_devices_queue_execution.connect("bind",bind_expander)

                factory_devices_queue_execution_value = Gtk.SignalListItemFactory()
                factory_devices_queue_execution_value.connect("setup",setup)
                factory_devices_queue_execution_value.connect("bind",bind1)

                deviceQueueExecutionSelection = Gtk.SingleSelection()
                # Store already initialized at top

                deviceQueueExectionModel = Gtk.TreeListModel.new(DeviceQueueExecution_store,False,True,add_tree_node)
                deviceQueueExecutionSelection.set_model(deviceQueueExectionModel)

                deviceQueueExecutionColumnView.set_model(deviceQueueExecutionSelection)

                deviceQueueExectionColumnLhs = Gtk.ColumnViewColumn.new("Device Information",factory_devices_queue_execution)
                deviceQueueExectionColumnLhs.set_resizable(True)
                deviceQueueExectionColumnRhs = Gtk.ColumnViewColumn.new("Details",factory_devices_queue_execution_value)
                deviceQueueExectionColumnRhs.set_expand(True)

                deviceQueueExecutionColumnView.append_column(deviceQueueExectionColumnLhs)
                deviceQueueExecutionColumnView.append_column(deviceQueueExectionColumnRhs)

                DeviceQueueExecutionScrollbar = create_scrollbar(deviceQueueExecutionColumnView)

                content_box.append(DeviceQueueExecutionScrollbar)
        elif "Device Vector" in tab_name:
                deviceVectorColumnView = Gtk.ColumnView()
                deviceVectorColumnView.props.show_row_separators = True
                deviceVectorColumnView.props.show_column_separators = False

                factory_devices_vector = Gtk.SignalListItemFactory()
                factory_devices_vector.connect("setup",setup_expander)
                factory_devices_vector.connect("bind",bind_expander)

                factory_devices_vector_value = Gtk.SignalListItemFactory()
                factory_devices_vector_value.connect("setup",setup)
                factory_devices_vector_value.connect("bind",bind1)

                deviceVectorSelection = Gtk.SingleSelection()
                # Store already initialized at top

                deviceVectorModel = Gtk.TreeListModel.new(DeviceVector_store,False,True,add_tree_node)
                deviceVectorSelection.set_model(deviceVectorModel)

                deviceVectorColumnView.set_model(deviceVectorSelection)

                deviceVectorColumnLhs = Gtk.ColumnViewColumn.new("Device Information",factory_devices_vector)
                deviceVectorColumnLhs.set_resizable(True)
                deviceVectorRhs = Gtk.ColumnViewColumn.new("Details",factory_devices_vector_value)
                deviceVectorRhs.set_expand(True)

                deviceVectorColumnView.append_column(deviceVectorColumnLhs)
                deviceVectorColumnView.append_column(deviceVectorRhs)

                DeviceVectorScrollbar = create_scrollbar(deviceVectorColumnView)

                content_box.append(DeviceVectorScrollbar)



    # Add the tabs to the sidebar ListBox
        content_stack.add_titled(content_box, tab_name.replace(" ", "-").lower(), tab_name)

    sidebar_scrolled_window = Gtk.ScrolledWindow.new()
    sidebar_scrolled_window.set_child(sidebar_listbox)

    def on_row_activated(listbox, row):
        # Get the label from the activated row to find the tab's name
        tab_label = row.get_child()
        tab_name = tab_label.get_label()
        
        # The name for the stack child is the tab name, with spaces replaced by dashes and lowercase
        child_name = tab_name.replace(" ", "-").lower()

        # Set the visible child of the stack using its name
        content_stack.set_visible_child_name(child_name)
        
    sidebar_listbox.connect("row-activated", on_row_activated)
    sidebar_listbox.select_row(sidebar_listbox.get_row_at_index(0))

    sidebar_page = Adw.NavigationPage.new(sidebar_scrolled_window, "Sidebar")
    content_page = Adw.NavigationPage.new(content_stack, "Content")
    
    # Set the sidebar and content for the split view using the new pages
    split_view.set_sidebar(sidebar_page)
    split_view.set_content(content_page)
    
    # Add the split view to its new container
    split_view_container.append(split_view)
    
    # The Platform Drop Down

    h_box = Gtk.Box.new(orientation=Gtk.Orientation.HORIZONTAL, spacing=10)
    h_box.set_halign(Gtk.Align.CENTER)
    setMargin(h_box,10,10,10)

#    platformGrid = Gtk.Grid()
 #   platformGrid.set_border_width(20)
#    platformGrid.set_column_spacing(20)
#    platformGrid.set_row_spacing(10)
    #   mainGrid.set_row_spacing(10)
#    platformFrame = Gtk.Frame(hexpand=True)
#    mainGrid.attach(platformFrame,0,0,1,1)
 #   platformFrame.set_child(platformGrid)

    platformLabel = Gtk.Label()
    platformLabel.add_css_class(css_class="toolbar")
#    setMargin(platformLabel,300,10,10)
    platformLabel.set_text("Available Platform(s):")
 #   platformGrid.attach(platformLabel, 0, 1, 1, 1)

    h_box.append(platformLabel)

#    h_device_box = Gtk.Box.new(orientation=Gtk.Orientation.HORIZONTAL, spacing=2)
#    h_device_box.set_halign(Gtk.Align.START)
#    setMargin(h_device_box,10,10,10)
    platform_list = Gtk.StringList()

    platform_names = getPlatformNames()

    for i in platform_names:
        platform_list.append(i)

    platform_dropdown = Gtk.DropDown()
    self.opencl_platform_dropdown = platform_dropdown
    platform_dropdown.add_css_class(css_class="card")
    platform_dropdown.set_model(platform_list)

    platform_dropdown.connect("notify::selected-item",selectPlatform)

    h_box.append(platform_dropdown)

    AvailableDevices = Gtk.Label()
    
    AvailableDevices.set_label("Available Device(s) :")
    setMargin(AvailableDevices,50,10,10)
    h_box.append(AvailableDevices)
#    platformGrid.attach_next_to(AvailableDevices, platformLabel, Gtk.PositionType.BOTTOM, 2, 1)

    Devices_list = Gtk.StringList()
    Devices_dropdown = Gtk.DropDown()
    self.opencl_devices_dropdown = Devices_dropdown
    Devices_dropdown.add_css_class(css_class="card")
    Devices_dropdown.set_model(Devices_list)
    setMargin(Devices_dropdown,20,10,10)
    Devices_dropdown.connect("notify::selected-item",selectDevice)
    h_box.append(Devices_dropdown)

    gpu_image = Gtk.Image()
    gpu_image = GdkPixbuf.Pixbuf.new_from_file_at_size(const.APP_LOGO_PNG, 100, 100)
    image_renderer = Gtk.Picture.new_for_pixbuf(gpu_image)
    Devices_dropdown.set_margin_end(10)
    h_box.append(image_renderer)

#    platformGrid.attach_next_to(Devices_dropdown, AvailableDevices, Gtk.PositionType.RIGHT, 20, 1)

    numberOfDevicesEntry = Gtk.Entry()
    setMargin(numberOfDevicesEntry,30,10,10)

    setMargin(platform_dropdown,30,10,10)
    selectPlatform(platform_dropdown,0)


#    platformGrid.attach_next_to(platform_dropdown, platformLabel, Gtk.PositionType.RIGHT, 21, 1)

    numberOfPlatforms = Gtk.Label()
    setMargin(numberOfPlatforms,30,10,10)
    numberOfPlatforms.set_label("Number of Platforms :")
#    h_platform_box.append(numberOfPlatforms)
#    platformGrid.attach_next_to(numberOfPlatforms, platform_dropdown, Gtk.PositionType.RIGHT, 1, 1)

    numberOfPlatformsEntry = Gtk.Entry()
    setMargin(numberOfPlatformsEntry,30,10,10)
    numberOfPlatformsEntry.set_text(str(len(oclPlatforms)))
    numberOfPlatformsEntry.set_editable(False)
#    h_platform_box.append(numberOfPlatformsEntry)
#    platformGrid.attach_next_to(numberOfPlatformsEntry, numberOfPlatforms, Gtk.PositionType.RIGHT, 1, 1)

    numberOfDevices = Gtk.Label()
    setMargin(numberOfDevices,30,10,10)
    numberOfDevices.set_label("Number Of Devices :")
#    h_device_box.append(numberOfDevices)
#    platformGrid.attach_next_to(numberOfDevices, Devices_dropdown, Gtk.PositionType.RIGHT, 1, 1)

    numberOfDevicesEntry.set_max_length(2)

#    platformGrid.attach_next_to(numberOfDevicesEntry, numberOfDevices, Gtk.PositionType.RIGHT, 1, 1)
    tab.append(h_box)
    tab.append(split_view_container)
    return tab


#!/usr/bin/env python3

# Copyright (C) 2017-2026 Arun Sivaraman <arunsivaramanneo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

# Import necessary libraries.
import sys, os

os.environ["PYTHONSAFEPATH"] = "1"
script_dir = os.path.dirname(os.path.abspath(__file__))
if script_dir not in sys.path:
    sys.path.insert(0, script_dir)
os.chdir(script_dir)

import gi
import subprocess
import shutil
import threading
from concurrent.futures import ThreadPoolExecutor, as_completed

# This line is crucial for working with PyGObject.
# It ensures we are using the correct versions of the libraries.
gi.require_version("Gtk", "4.0")
gi.require_version("Adw", "1")
from gi.repository import Gtk, Adw, GObject, Gdk, Gio, GLib
from vulkan_viewer import create_vulkan_tab_content
import Filenames, const
from Common import copyContentsFromFile,getScreenSize,fetchContentsFromCommand,setMargin,Config
from aboutPage import about_page
from OpenGLViewer import OpenGL
from OpenCL import openCL
from VdpauViewer import vdpauinfo
from VulkanVideoViewer import VulkanVideo
from pathlib import Path

# Define the main application class.
# It inherits from Adw.Application, which provides a modern application shell.

def isVulkanSupported():
    # Runs inside ThreadPoolExecutor (parallel with glxinfo/clinfo/vdpauinfo),
    # so --show-formats completes fully before _build_tabs_on_main_thread is
    # called — no race condition against create_vulkan_tab_content().
    with open(Filenames.vulkaninfo_output_file, "w") as file:
        vulkan_process = subprocess.Popen(
            Filenames.vulkaninfo_output_command,
            shell=True, stdout=file, universal_newlines=True
        )
        vulkan_process.wait()
        vulkan_process.communicate()
    vulkaninfo_output = copyContentsFromFile(Filenames.vulkaninfo_output_file)
    return len(vulkaninfo_output) > 10 and vulkan_process.returncode == 0

def isOpenglSupported():
    with open(Filenames.opengl_outpuf_file, "w") as file:
        opengl_process = subprocess.Popen(
            Filenames.opengl_output_command,
            shell=True, stdout=file, universal_newlines=True
        )
        opengl_process.wait()
        opengl_process.communicate()
    opengl_output = copyContentsFromFile(Filenames.opengl_outpuf_file)
    return len(opengl_output) > 10 and opengl_process.returncode == 0

def isOpenclSupported():
    # Fix #3 (part 1): Write clinfo output to the shared file so OpenCL.py
    # can reuse it without spawning a second clinfo process.
    with open(Filenames.opencl_output_file, "w") as file:
        clinfo_process = subprocess.Popen(
            "clinfo",
            shell=True, stdout=file, universal_newlines=True
        )
        clinfo_process.wait()
        clinfo_process.communicate()
    clinfo_output = copyContentsFromFile(Filenames.opencl_output_file)
    return len(clinfo_output) > 10 and clinfo_process.returncode == 0

def isVdpauinfoSupported():
    with open(Filenames.vdpauinfo_output_file, "w") as file:
        vdpauinfo_process = subprocess.Popen(
            Filenames.vdpauinfo_output_command,
            shell=True, stdout=file, universal_newlines=True
        )
        vdpauinfo_process.wait()
        vdpauinfo_process.communicate()
    vdpauinfo_output = copyContentsFromFile(Filenames.vdpauinfo_output_file)
    return len(vdpauinfo_output) > 10 and vdpauinfo_process.returncode == 0

def isVulkanVideoSupported():
    # Fix #1 & #6: Grep the file already written by isVulkanSupported() —
    # no need to spawn a second vulkaninfo process.
    try:
        return len(fetchContentsFromCommand(
            f"grep 'Video Profiles' {Filenames.vulkaninfo_output_file}"
        )) > 0
    except Exception:
        return False

def quit(instance):
    unset_lc_all_process = subprocess.Popen(Filenames.unset_LC_ALL_conmand,stdout=subprocess.PIPE,stderr=subprocess.PIPE,shell=True)
    unset_lc_all_process.communicate()
    rmdir_process =subprocess.Popen(Filenames.rmdir_output_command,stdout=subprocess.PIPE,stderr=subprocess.PIPE,shell=True)
    rmdir_process.communicate()
    instance.destroy()

def simple_quit(instance):
    instance.destroy()

if Path(Filenames.gpu_viewer_folder_path).exists():
    
    def show_message(app):
        dialog = Gtk.AlertDialog()
        dialog.set_modal(False)
        dialog.set_message('gpu-viewer is already running')
        dialog.set_detail('If you are unable to view the application, please run rm -r /tmp/gpu-viewer and run the application again')
    #    dialog.set_default_button(0)
        dialog.set_cancel_button(1)
        dialog.choose(None,None,None,None)
    #    dialog.show()
        message_window = Gtk.ApplicationWindow(application=app)
        message_grid = Gtk.Grid()
        message_window.set_title("gpu-viewer application is already running")
        message_window.set_default_size(480,120)
        message_window.set_resizable(False)
    #    message_window.present()
        message_window_frame = Gtk.Frame()
        setMargin(message_window_frame,5,5,10)
        label = Gtk.Label(label="If you are unable to view the application, please run rm -r /tmp/gpu-viewer and run the application again")
        message_window.set_child(message_window_frame)
        message_window_frame.set_child(message_grid)
        setMargin(label,5,10,0)
        message_grid.attach(label,0,0,20,1)
        
        message_button_OK = Gtk.Button.new_with_label("OK")
        message_button_OK.connect("clicked",simple_quit)
    #    message_button_CANCEL = Gtk.Button.new_with_label("No")
        setMargin(message_button_OK,500,50,10)
        message_grid.attach_next_to(message_button_OK,label,Gtk.PositionType.BOTTOM,5,1)
     #   message_grid.attach_next_to(message_button_CANCEL,message_button_OK,Gtk.PositionType.RIGHT,10,1)
    #    setMargin(message_button_CANCEL,50,50,10)
        setMargin(message_window,5,5,10)

    app = Gtk.Application()
    app.connect("activate",show_message)
    app.run(None)


else:
    class GPUViewerApp(Adw.Application):
        """
        A simple Libadwaita application class.
        Inherits from Adw.Application to provide the application framework.
        """
        def __init__(self, **kwargs):
            # Call the parent constructor, providing a unique application ID
            super().__init__(application_id="io.github.arunsivaramanneo.GPUViewer", **kwargs)
            self.config = Config()
            self.connect("activate", self.on_activate)    

        def _on_save_button_clicked(self, button):
            visible_child_name = self.view_stack.get_visible_child_name()
            
            source_file = None
            default_name = "output.txt"
            
            if visible_child_name == "page1":
                source_file = Filenames.vulkaninfo_output_file
                default_name = "vulkaninfo.txt"
                if hasattr(self, 'vulkan_content_stack'):
                    vulkan_tab = self.vulkan_content_stack.get_visible_child_name()
                    if vulkan_tab == "system-information":
                        source_file = Filenames.vulkaninfo_output_file
                        default_name = "vulkaninfo.txt"
                    elif vulkan_tab == "device":
                        source_file = Filenames.vulkan_device_info_file
                        default_name = "deviceinfo.txt"
                    elif vulkan_tab == "properties":
                        source_file = Filenames.vulkan_device_properties_file
                        default_name = "properties.txt"
                    elif vulkan_tab == "limits":
                        source_file = Filenames.vulkan_device_limits_file
                        default_name = "limits.txt"
                    elif vulkan_tab == "features":
                        source_file = Filenames.vulkan_device_features_file
                        default_name = "features.txt"
                    elif vulkan_tab == "formats":
                        source_file = Filenames.vulkan_device_formats_file
                        default_name = "formats.txt"
                    elif vulkan_tab == "memory-types":
                        source_file = Filenames.vulkan_device_memory_types_file
                        default_name = "memory_types.txt"
                    elif vulkan_tab == "memory-heaps":
                        source_file = Filenames.vulkan_device_memory_heaps_file
                        default_name = "memory_heaps.txt"
                    elif vulkan_tab == "queues":
                        source_file = Filenames.vulkan_device_queues_file
                        default_name = "queues.txt"
                    elif vulkan_tab == "instance-extensions":
                        source_file = Filenames.vulkan_device_extensions_file
                        default_name = "extensions.txt"
                    elif vulkan_tab == "instance-layers":
                        source_file = Filenames.vulkan_device_layers_file
                        default_name = "layers.txt"
                    elif vulkan_tab == "surface":
                        source_file = Filenames.vulkan_device_surface_file
                        default_name = "surface.txt"

            elif visible_child_name == "page2":
                source_file = Filenames.opengl_outpuf_file
                default_name = "glxinfo.txt"
                if hasattr(self, 'opengl_extensions_notebook'):
                    opengl_tab = self.opengl_extensions_notebook.get_visible_child_name()
                    if opengl_tab == "opengl":
                        pass
                    elif opengl_tab == "opengl_es":
                        source_file = Filenames.opengl_vendor_es_extension_file
                        default_name = "opengl_es_extensions.txt"
                    elif opengl_tab == "egl_logo":
                        source_file = Filenames.egl_vendor_extension_file
                        default_name = "egl_extensions.txt"

            elif visible_child_name == "opencl_page":
                source_file = Filenames.opencl_output_file
                default_name = "clinfo.txt"
                if hasattr(self, 'opencl_content_stack'):
                    opencl_tab = self.opencl_content_stack.get_visible_child_name()
                    props = None
                    if hasattr(self, 'opencl_platforms') and self.opencl_platforms:
                        platform_idx = self.opencl_platform_dropdown.props.selected if hasattr(self, 'opencl_platform_dropdown') else 0
                        if platform_idx == Gtk.INVALID_LIST_POSITION:
                            platform_idx = 0
                        device_selected = self.opencl_devices_dropdown.props.selected if hasattr(self, 'opencl_devices_dropdown') else 0
                        if device_selected != Gtk.INVALID_LIST_POSITION:
                            device_idx = self.opencl_gpu_index_map[device_selected] if (hasattr(self, 'opencl_gpu_index_map') and len(self.opencl_gpu_index_map) > device_selected) else device_selected
                        else:
                            device_idx = 0
                        if opencl_tab == "platform-information":
                            if platform_idx < len(self.opencl_platforms):
                                props = self.opencl_platforms[platform_idx]["properties"]
                                default_name = "platform_details.txt"
                        elif hasattr(self, 'opencl_get_device_categories'):
                            cats = self.opencl_get_device_categories(platform_idx, device_idx)
                            if cats:
                                if opencl_tab == "device-information":
                                    props = cats["Device Information"]
                                    default_name = "device_details.txt"
                                elif opencl_tab == "device-extensions":
                                    props = cats["Device Extensions with Version"]
                                    default_name = "device_extensions.txt"
                                elif opencl_tab and (opencl_tab.startswith("device-memory") or "image" in opencl_tab):
                                    props = cats["Device Memory & Image Information"]
                                    default_name = "memory_and_image_details.txt"
                                elif opencl_tab and (opencl_tab.startswith("queue-capabilities") or "queue" in opencl_tab):
                                    props = cats["Queue & Execution Capabilities"]
                                    default_name = "queue_execution_details.txt"
                                elif opencl_tab == "device-vector-information":
                                    props = cats["Device Vector Information"]
                                    default_name = "vector_details.txt"
                    if props is not None:
                        temp_file = "/tmp/gpu-viewer/opencl_tab_output.txt"
                        try:
                            with open(temp_file, "w") as f:
                                for key, value, children in props:
                                    if value:
                                        f.write(f"{key}: {value}\n")
                                    else:
                                        f.write(f"{key}\n")
                                    for sub_key, sub_val in children:
                                        f.write(f"    {sub_key}: {sub_val}\n")
                            source_file = temp_file
                        except Exception as e:
                            print(f"Error writing temporary OpenCL file: {e}")

            elif visible_child_name == "vdpau_page":
                source_file = Filenames.vdpauinfo_output_file
                default_name = "vdpauinfo.txt"
            elif visible_child_name == "vulkan_video_page":
                source_file = "/tmp/gpu-viewer/vulkan_video_info.txt"
                default_name = "vulkan_video_info.txt"
                with open(source_file, "w") as f:
                    subprocess.run(["vulkaninfo", "--show-video-props"], stdout=f, universal_newlines=True)
            else:
                return
            
            if not source_file or not os.path.exists(source_file):
                return

            dialog = Gtk.FileChooserNative.new("Save Tab Output",
                                               self.window,
                                               Gtk.FileChooserAction.SAVE,
                                               "Save",
                                               "Cancel")
            dialog.set_current_name(default_name)
            
            def on_response(native, response):
                if response == Gtk.ResponseType.ACCEPT:
                    file_out = native.get_file()
                    if file_out:
                        dest_path = file_out.get_path()
                        if dest_path:
                            shutil.copyfile(source_file, dest_path)
                            
            dialog.connect("response", on_response)
            dialog.show()

        def _on_theme_button_clicked(self, button):
            """
            Callback to handle the theme button click.
            It toggles the Adw.StyleManager's color scheme.
            """
            self.theme_button.set_sensitive(False)
            prefer_dark_theme = not self.config.get_theme_preference()
            self._apply_theme(prefer_dark_theme)

        def _apply_theme(self, state):
            """
            Applies the theme based on the state (True for dark, False for light).
            """
            style_manager = Adw.StyleManager.get_default()
            self.config.set_theme_preference(state)
            
            if state:
                # Set to dark theme
                style_manager.set_color_scheme(Adw.ColorScheme.FORCE_DARK)
                fname = Gio.file_new_for_path('gtk_dark.css')
            else:
                # Set to light theme
                style_manager.set_color_scheme(Adw.ColorScheme.FORCE_LIGHT)
                fname = Gio.file_new_for_path('gtk_light.css')

            # Reload CSS
            display = Gtk.Widget.get_display(self.window)
            provider = Gtk.CssProvider.new()
            provider.load_from_file(fname)
            Gtk.StyleContext.add_provider_for_display(display, provider,
                Gtk.STYLE_PROVIDER_PRIORITY_USER)

            # Update constants
            const.update_theme_constants(state)

            # Update button icon
            self._update_theme_button_icon(state)
            
            # Refresh UI by re-creating tabs
            self.refresh_tabs()

        def _update_theme_button_icon(self, is_dark):
            """Updates the icon of the theme toggle button."""
            if is_dark:
                self.theme_icon.set_from_icon_name("display-brightness-symbolic")
            else:
                self.theme_icon.set_from_icon_name("weather-clear-night-symbolic")


        def on_activate(self, app):
            """
            Callback function for the 'activate' signal.
            This is where we create and show the main window.
            """
            # Create a new Adwaita ApplicationWindow
            self.window = Adw.ApplicationWindow.new(self)
            self.window.add_css_class(css_class="compact")
            self.window.set_title("GPU-Viewer v3.2")
            self.window.add_css_class(css_class="view")

            # Set the application's default size to 800x800

            width,height = getScreenSize()

            if int(width) > 2160 and int(height) < 1440:
                self.window.set_size_request(2160 * const.WIDTH_RATIO ,int(height) * const.HEIGHT_RATIO1)
            elif int(width) > 2160 and int(height) > 1440:
                self.window.set_size_request(2160 * const.WIDTH_RATIO ,1440 * const.HEIGHT_RATIO1)
            else:
                self.window.set_size_request(int(width) * const.WIDTH_RATIO ,int(height) * const.HEIGHT_RATIO1)
    
            provider = Gtk.CssProvider.new()
            style_manager = Adw.StyleManager.get_default()
            prefer_dark_theme = self.config.get_theme_preference()
            
            if prefer_dark_theme:
                style_manager.set_color_scheme(Adw.ColorScheme.FORCE_DARK)
                fname = Gio.file_new_for_path('gtk_dark.css')
            else:
                style_manager.set_color_scheme(Adw.ColorScheme.FORCE_LIGHT)
                fname = Gio.file_new_for_path('gtk_light.css')

            display = Gtk.Widget.get_display(self.window)
            provider.load_from_file(fname)
            Gtk.StyleContext.add_provider_for_display(display, provider,
                Gtk.STYLE_PROVIDER_PRIORITY_USER)
            
            # Synchronize constants with the loaded preference
            const.update_theme_constants(prefer_dark_theme)
            # Create a new Adwaita view stack to hold the different pages
            self.view_stack = Adw.ViewStack.new()

            # Create a view switcher that will display icons
            self.switcher = Adw.ViewSwitcher.new()
            self.switcher.set_stack(self.view_stack)
            self.switcher.set_policy(1)
            # Create a header bar
            self.header_bar = Adw.HeaderBar.new()
        #    self.header_bar.add_css_class(css_class="inline")
            # Set the view switcher as the custom title widget in the header bar
            self.header_bar.add_css_class(css_class="view")

            icon_theme = Gtk.IconTheme.get_for_display(Gdk.Display.get_default())
            icon_theme.add_search_path(os.path.abspath("../Images"))

            # Create the theme toggle button
            self.theme_button = Gtk.Button.new()
            self.theme_button.add_css_class("flat")
            self.theme_button.set_valign(Gtk.Align.CENTER)
            self.theme_button.set_sensitive(False)
            
            self.theme_icon = Gtk.Image.new()
            self._update_theme_button_icon(prefer_dark_theme)
            self.theme_button.set_child(self.theme_icon)
            
            # Connect the button's 'clicked' signal
            self.theme_button.connect("clicked", self._on_theme_button_clicked)

            # Create the save button
            self.save_button = Gtk.Button.new()
            self.save_button.add_css_class("flat")
            self.save_button.set_valign(Gtk.Align.CENTER)
            self.save_button.set_icon_name("document-save-symbolic")
            self.save_button.connect("clicked", self._on_save_button_clicked)

            # Create a box to hold the buttons
            header_buttons_box = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 5)
            header_buttons_box.set_halign(Gtk.Align.END)
            header_buttons_box.set_valign(Gtk.Align.CENTER)
            header_buttons_box.append(self.save_button)
            header_buttons_box.append(self.theme_button)
            self.header_bar.pack_end(header_buttons_box)
            self.header_bar.set_title_widget(title_widget=self.switcher)

            # Create a main box to hold the header bar and the view stack
            main_box = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
            main_box.append(self.header_bar)
            main_box.append(self.view_stack)



            # Set the main box as the content of the window
            self.window.set_content(main_box)
            self.window.connect("close-request", quit)

            # Fix #4: Show the window immediately so the user sees it right
            # away, then kick off tab-building asynchronously.
            self.show_window()
            GLib.idle_add(self._start_loading_tabs)

        def refresh_tabs(self):
            """Clears and re-creates all tabs in the view stack."""
            # Remove all current children
            child = self.view_stack.get_first_child()
            while child:
                next_child = child.get_next_sibling()
                self.view_stack.remove(child)
                child = next_child
            # Re-show spinner and re-trigger async load
            self._start_loading_tabs()

        def _start_loading_tabs(self):
            """Show a loading spinner immediately, then build tabs in a background thread."""
            # Show a spinner page so the user sees *something* right away
            self._loading_box = Gtk.Box.new(Gtk.Orientation.VERTICAL, 12)
            self._loading_box.set_valign(Gtk.Align.CENTER)
            self._loading_box.set_halign(Gtk.Align.CENTER)
            self._loading_box.set_vexpand(True)

            spinner = Gtk.Spinner()
            spinner.set_size_request(48, 48)
            spinner.start()

            loading_label = Gtk.Label(label="Loading GPU information…")
            loading_label.add_css_class("dim-label")

            self._loading_box.append(spinner)
            self._loading_box.append(loading_label)
            self.view_stack.add_titled_with_icon(self._loading_box, "loading", "Loading", "system-run-symbolic")

            # Ensure the /tmp/gpu-viewer directory exists before threads start
            subprocess.Popen(Filenames.mkdir_output_command, stdout=subprocess.PIPE, shell=True).communicate()

            # Fix #2: Run all support checks in parallel, then build UI on main thread
            thread = threading.Thread(target=self._probe_and_build_tabs, daemon=True)
            thread.start()
            return False  # Do not repeat GLib.idle_add

        def _probe_and_build_tabs(self):
            """Run support checks in parallel (background thread), then schedule UI build."""
            # Fix #2: parallel support checks via ThreadPoolExecutor
            with ThreadPoolExecutor(max_workers=4) as executor:
                futures = {
                    executor.submit(isVulkanSupported):   "vulkan",
                    executor.submit(isOpenglSupported):   "opengl",
                    executor.submit(isOpenclSupported):   "opencl",
                    executor.submit(isVdpauinfoSupported): "vdpau",
                }
                results = {}
                for future in as_completed(futures):
                    key = futures[future]
                    try:
                        results[key] = future.result()
                    except Exception as e:
                        print(f"Support check failed for {key}: {e}")
                        results[key] = False

            # Fix #1 & #6: vulkan-video check reuses the file written by isVulkanSupported
            results["vulkan_video"] = isVulkanVideoSupported() if results.get("vulkan") else False

            # Schedule the actual GTK widget construction back on the main thread
            GLib.idle_add(self._build_tabs_on_main_thread, results)

        def _build_tabs_on_main_thread(self, results):
            """Called on the GLib main thread — safe to create/modify GTK widgets.

            Each tab is immediately added to the outer ViewStack with a fixed inner
            Gtk.Stack that starts on its "spinner" child.  When the real content is
            ready the inner stack simply switches to its "content" child — the outer
            ViewStack page never moves, so tabs never jump or reorder.
            """
            # Remove the global loading spinner page
            if hasattr(self, '_loading_box') and self._loading_box:
                self.view_stack.remove(self._loading_box)
                self._loading_box = None

            self._tab_built = {}          # page_name → bool
            self._inner_stacks = {}       # page_name → inner Gtk.Stack

            def _make_placeholder(label_text):
                """Centred spinner displayed until the real content arrives."""
                box = Gtk.Box.new(Gtk.Orientation.VERTICAL, 12)
                box.set_valign(Gtk.Align.CENTER)
                box.set_halign(Gtk.Align.CENTER)
                box.set_vexpand(True)
                sp = Gtk.Spinner()
                sp.set_size_request(36, 36)
                sp.start()
                lbl = Gtk.Label(label=f"Loading {label_text}…")
                lbl.add_css_class("dim-label")
                box.append(sp)
                box.append(lbl)
                return box, sp  # return spinner ref so we can stop it later

            def _register_tab(page_name, title, icon, builder):
                """Add a stable ViewStack page with an inner stack (spinner + content)."""
                inner = Gtk.Stack()
                inner.set_vexpand(True)
                inner.set_hexpand(True)
                # Disable transition animation inside the inner stack
                inner.set_transition_type(Gtk.StackTransitionType.NONE)

                placeholder, spinner_widget = _make_placeholder(title)
                inner.add_named(placeholder, "spinner")
                # "content" child is added later by _reveal_content

                # The outer ViewStack page wraps the inner stack — it never moves
                self.view_stack.add_titled_with_icon(inner, page_name, title, icon)
                self._inner_stacks[page_name] = (inner, spinner_widget)
                self._tab_built[page_name] = False

            def _build_tab_async(page_name):
                """Run builder in a worker thread; reveal result on main thread."""
                builder = self._tab_builders[page_name]
                def _worker():
                    real_widget = builder()
                    GLib.idle_add(_reveal_content, page_name, real_widget)
                threading.Thread(target=_worker, daemon=True).start()

            def _reveal_content(page_name, real_widget):
                """Swap spinner → real content inside the inner stack (main thread)."""
                entry = self._inner_stacks.get(page_name)
                if entry is None:
                    return False
                inner, spinner_widget = entry
                inner.add_named(real_widget, "content")
                inner.set_visible_child_name("content")
                # Stop the spinner to free resources
                spinner_widget.stop()
                self._tab_built[page_name] = True
                return False  # Don't repeat

            # Keep builder callables in a separate dict (needed by _build_tab_async)
            self._tab_builders = {}

            def _register(page_name, title, icon, builder):
                self._tab_builders[page_name] = builder
                _register_tab(page_name, title, icon, builder)

            # ---- Register all supported tabs ----
            if results.get("vulkan"):
                _register("page1", "Vulkan", "Vulkan",
                           lambda: create_vulkan_tab_content(self))

            if results.get("opengl"):
                def _build_opengl():
                    box = Gtk.Box.new(Gtk.Orientation.VERTICAL, 10)
                    return OpenGL(self, box)
                _register("page2", "OpenGL", "OpenGL", _build_opengl)

            if results.get("opencl"):
                def _build_opencl():
                    box = Gtk.Box.new(Gtk.Orientation.VERTICAL, 10)
                    return openCL(self, box)
                _register("opencl_page", "OpenCL", "OpenCL", _build_opencl)

            if results.get("vulkan_video"):
                def _build_vulkan_video():
                    box = Gtk.Box.new(Gtk.Orientation.VERTICAL, 10)
                    return VulkanVideo(box)
                _register("vulkan_video_page", "Vulkan Video", "Vulkan-Video",
                           _build_vulkan_video)

            if results.get("vdpau"):
                def _build_vdpau():
                    box = Gtk.Box.new(Gtk.Orientation.VERTICAL, 10)
                    return vdpauinfo(box)
                _register("vdpau_page", "VDPAU", "vdpauinfo", _build_vdpau)

            # About page has no subprocesses — build immediately
            page3_box = Gtk.Box.new(Gtk.Orientation.VERTICAL, 10)
            self.about_page_ui = about_page(page3_box)
            self.view_stack.add_titled_with_icon(page3_box, "page3", "About Us", "about-us")

            # ---- Trigger a lazy build whenever the visible tab changes ----
            def _on_visible_child_changed(stack, _pspec):
                name = stack.get_visible_child_name()
                if name and not self._tab_built.get(name, True):
                    self._tab_built[name] = True   # prevent double-build
                    _build_tab_async(name)

            self.view_stack.connect("notify::visible-child", _on_visible_child_changed)

            # Start building the first (default) tab automatically
            first = self.view_stack.get_visible_child_name()
            if first and not self._tab_built.get(first, True):
                self._tab_built[first] = True
                _build_tab_async(first)

            if hasattr(self, 'theme_button') and self.theme_button:
                self.theme_button.set_sensitive(True)

            return False  # Do not repeat GLib.idle_add

        def show_window(self):
            # Show the window and all its children
            self.window.present()


    def main():
        """
        The main entry point of the application.
        """
        # Create an instance of our application class
        app = GPUViewerApp()

        # Run the application
        app.run(sys.argv)


    if __name__ == "__main__":
        main()



# Copyright (C) 2017-2026 Arun Sivaraman <arunsivaramanneo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import gi
import const
import Filenames
import subprocess
from Common import ExpandDataObject, setup_expander,bind_expander,setup,bind1,add_tree_node, ExpandDataObject2,add_tree_node2,bind2,bind3,bind4,setMargin

gi.require_version("Gtk", "4.0")
from gi.repository import Gtk,GObject,Gio,Adw
Adw.init()

from Common import createSubTab, create_scrollbar

class DataObject(GObject.GObject):
    def __init__(self, column1: str,column2: str,column3: str,column4: str,column5: str):
        super(DataObject, self).__init__()
        self.column1 = column1
        self.column2 = column2
        self.column3 = column3
        self.column4 = column4
        self.column4 = column5

def vdpauinfo(tab2):

	def vdpauInformation():

		fetch_vdpau_information_command = "cat %s | grep -E 'API|string' " %(Filenames.vdpauinfo_output_file)
		fetch_Vdpau_information_process = subprocess.Popen(fetch_vdpau_information_command,shell=True,stdout=subprocess.PIPE,universal_newlines=True)
		vdpauinfo_information = fetch_Vdpau_information_process.communicate()[0].splitlines()

		for i,line in enumerate(vdpauinfo_information):
		#	vdpauinfoStore.append(None,[' '.join(line.split()[0:2]).strip('[]'),' '.join(line.split()[2:]).strip('[]'),setBackgroundColor(i)])
			toprow = ExpandDataObject(' '.join(line.split()[0:2]).strip('[]'),' '.join(line.split()[2:]).strip('[]'))
			vdpauinfoStore.append(toprow)	  
		#iter = vdpauinfoStore.append(None,["Supported Codecs:",' ',const.BGCOLOR3])
		
		iter = ExpandDataObject("Supported Codecs:",' ')
	#	vdpauinfoStore.append(iter)

		fetch_vdpau_decoder_capabilities_command = "cat %s | awk '/Decoder capabilities:/{flag=1;next}/Output surface:/{flag=0}flag' | awk '/------.*/{flag=1;next}/Output surface:/{flag=0}flag' | awk '/./'" %(Filenames.vdpauinfo_output_file)
		fetch_vdpau_decoder_capabilities_process = subprocess.Popen(fetch_vdpau_decoder_capabilities_command,shell=True,stdout=subprocess.PIPE,universal_newlines=True)
		vdpau_decoder_capabilities = fetch_vdpau_decoder_capabilities_process.communicate()[0].splitlines()
		
		i = 0
		for line in  vdpau_decoder_capabilities:
			if "not" not in line:
				iter2 = ExpandDataObject(line.split()[0].strip('\n')," ")
				iter.children.append(iter2)
			#	vdpauinfoStore.append(iter,[line.split()[0].strip('\n')," ",setBackgroundColor(i)])
				i = i + 1
		vdpauinfoStore.append(iter)

	#	treeVdpauInfo.expand_all()

	def decoderCapabilities():

		fetch_vdpau_decoder_capabilities_command = "cat %s | awk '/Decoder capabilities:/{flag=1;next}/Output surface:/{flag=0}flag' | awk '/------.*/{flag=1;next}/Output surface:/{flag=0}flag' | awk '/./'" %(Filenames.vdpauinfo_output_file)
		fetch_vdpau_decoder_capabilities_process = subprocess.Popen(fetch_vdpau_decoder_capabilities_command,shell=True,stdout=subprocess.PIPE,universal_newlines=True)
		vdpau_decoder_capabilities = fetch_vdpau_decoder_capabilities_process.communicate()[0].splitlines()

		for i,line in  enumerate(vdpau_decoder_capabilities):
			if "not" in line:
				decoderStore.append(ExpandDataObject2(line.split()[0].strip('\n'),"not supported","not supported","not supported","not supported"))
			else:
				decoderStore.append(ExpandDataObject2(line.split()[0].strip('\n'),line.split()[1].strip('\n'),line.split()[2].strip('\n'),line.split()[3].strip('\n'),line.split()[4].strip('\n')))
	
	def videoSurfaceLimits():
		
		fetch_vdpau_surface_limits_command = "cat %s | awk '/Video surface:/{flag=1;next}/Decoder capabilities:/{flag=0}flag' | awk '/-------.*/{flag=1;next}/Bitmap surface:/{flag=0}flag' | awk '/./'" %(Filenames.vdpauinfo_output_file)
		fetch_vdpau_surface_limits_process = subprocess.Popen(fetch_vdpau_surface_limits_command,shell=True,stdout=subprocess.PIPE,universal_newlines=True)
		vdpau_surface_limits = fetch_vdpau_surface_limits_process.communicate()[0].splitlines()
	
		toprow = ExpandDataObject2("Video Surface","","","","")
		for i,line in enumerate(vdpau_surface_limits):
			iter = ExpandDataObject2(line.split()[0].strip('\n'),line.split()[1].strip('\n'),line.split()[2].strip('\n'),(' '.join(line.split()[3:]).strip('[]')),"")
			toprow.children.append(iter)
		surfaceVideoStore.append(toprow)

		fetch_vdpau_surface_output_limits_command = "cat %s | awk '/Output surface:/{flag=1;next}/Bitmap surface:/{flag=0}flag' | awk '/-------.*/{flag=1;next}/Bitmap surface:/{flag=0}flag' | awk '/./' " %(Filenames.vdpauinfo_output_file)
		fetch_vdpau_surface_output_limits_process = subprocess.Popen(fetch_vdpau_surface_output_limits_command,shell=True,stdout=subprocess.PIPE,universal_newlines=True)
		vdpau_surface_output_limits = fetch_vdpau_surface_output_limits_process.communicate()[0].splitlines()

		toprow = ExpandDataObject2("Output Surface","","","","")
		for i,line in enumerate(vdpau_surface_output_limits):
			iter = ExpandDataObject2(line.split()[0].strip('\n'),line.split()[1].strip('\n'),line.split()[2],(' '.join(line.split()[4:]).strip('[]')),"")
			toprow.children.append(iter)
		surfaceVideoStore.append(toprow)

		fetch_vdpau_surface_bitmap_limits_command = "cat %s | awk '/Bitmap surface:/{flag=1;next}/Video mixer:/{flag=0}flag' | awk '/-------.*/{flag=1;next}/Video mixer:/{flag=0}flag' | awk '/./' " %(Filenames.vdpauinfo_output_file)
		fetch_vdpau_surface_bitmap_limits_process = subprocess.Popen(fetch_vdpau_surface_bitmap_limits_command,shell=True,stdout=subprocess.PIPE,universal_newlines=True)
		vdpau_surface_bitmap_limits = fetch_vdpau_surface_bitmap_limits_process.communicate()[0].splitlines()

		toprow = ExpandDataObject2("Bitmap Surface","","","","")
		for i,line in enumerate(vdpau_surface_bitmap_limits):
			iter = ExpandDataObject2(line.split()[0].strip('\n'),line.split()[1].strip('\n'),line.split()[2].strip('\n'),"","")
			toprow.children.append(iter)
		surfaceVideoStore.append(toprow)

		
	def VideoMixerFeature():
		
		fetch_vdpau_mixer_features_command = "cat %s | awk '/feature name/{flag=1;next}/parameter name.*/{flag=0}flag' | awk '/-------.*/{flag=1;next}/Output surface:/{flag=0}flag' | awk '/./' " %(Filenames.vdpauinfo_output_file)
		fetch_vdpau_mixer_features_process = subprocess.Popen(fetch_vdpau_mixer_features_command,shell=True,stdout=subprocess.PIPE,universal_newlines=True)
		vdpau_video_mixer_feature = fetch_vdpau_mixer_features_process.communicate()[0].splitlines()

		toprow = ExpandDataObject2("Features","","","","")
		for i,line in enumerate(vdpau_video_mixer_feature):
			if 'y'in line:
				text = "true"
				fgcolor = const.COLOR1
			else:
				text = "false"
				fgcolor = const.COLOR2
			if "HIGH" in line:
				iter = ExpandDataObject2((' '.join(line.split()[0:5])).strip('[]'),text,"","","")
				toprow.children.append(iter)
			else:
				iter = ExpandDataObject2(line.split()[0].strip('\n'),text,"","","")
				toprow.children.append(iter)
		videoMixerFeatureStore.append(toprow)
		
		fetch_vdpau_mixer_parameter_command = "cat %s | awk '/parameter name/{flag=1;next}/attribute name.*/{flag=0}flag' | awk '/-------.*/{flag=1;next}/Output surface:/{flag=0}flag' | awk '/./' " %(Filenames.vdpauinfo_output_file)
		fetch_vdpau_mixer_parameter_process = subprocess.Popen(fetch_vdpau_mixer_parameter_command,shell=True,stdout=subprocess.PIPE,universal_newlines=True)
		vdpau_video_mixer_parameters =  fetch_vdpau_mixer_parameter_process.communicate()[0].splitlines()

		toprow = ExpandDataObject2("Parameters","","","","")
		for i,line in enumerate(vdpau_video_mixer_parameters):
			if line.split()[1].strip('\n') == 'y':
				text = "true"
				fgcolor = const.COLOR1
			else:
				text = "false"
			iter = ExpandDataObject2(line.split()[0].strip('\n'),text,"","","")
			toprow.children.append(iter)

		videoMixerFeatureStore.append(toprow)		
		
		fetch_vdpau_mixer_attribute_command = "cat %s | awk '/Video mixer:/{flag=1;next}/Output surface:/{flag=0}flag' | awk '/attribute name/{flag=1;next}/Output surface:/{flag=0}flag' | awk '/-------.*/{flag=1;next}/Output surface:/{flag=0}flag' | awk '/./' " %(Filenames.vdpauinfo_output_file)
		fetch_vdpau_mixer_attribute_process = subprocess.Popen(fetch_vdpau_mixer_attribute_command,shell=True,stdout=subprocess.PIPE,universal_newlines=True)
		vdpau_video_mixer_attribute = fetch_vdpau_mixer_attribute_process.communicate()[0].splitlines()

		toprow = ExpandDataObject2("Attributes","","","","")
		for i,line in enumerate(vdpau_video_mixer_attribute):
			if line.split()[1].strip('\n') == 'y':
				text = "true"
				fgcolor = const.COLOR1
			else:
				text = "false"
			iter = ExpandDataObject2(line.split()[0].strip('\n'),text,"","","")
			toprow.children.append(iter)

		videoMixerFeatureStore.append(toprow	)

	

	def _on_search_method_changed(search_entry,filterColumn):
		filterColumn.changed(Gtk.FilterChange.DIFFERENT)

	def _do_filter_decoder_view(item, filter_list_model):
		search_text_widget = decoderSearchEntry.get_text()
		return search_text_widget.upper() in item.data.upper()

	split_view_container = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
	split_view_container.set_vexpand(True)
	split_view_container.set_hexpand(True)
    
    # Create a new NavigationSplitView
	split_view = Adw.NavigationSplitView.new()
	split_view.set_vexpand(True)
	split_view.set_hexpand(True)

    # Create the sidebar content using a Gtk.ListBox for tabs
	sidebar_listbox = Gtk.ListBox.new()
	sidebar_listbox.set_selection_mode(Gtk.SelectionMode.SINGLE)
	sidebar_listbox.set_vexpand(True)
	sidebar_listbox.add_css_class(css_class="boxed-list")
	sidebar_listbox.add_css_class(css_class="sidebar")


#    sidebar_listbox.add_css_class(css_class="frame")
    # Set the show-separators property to False
	sidebar_listbox.set_show_separators(True)
    
    # Add the tabs to the sidebar ListBox
	tabs = [
        "VDPAU Information", "Decoder Capabilities", "Surface Limits", "Video Mixer"
    ]

	content_stack = Gtk.Stack.new()

    # Create the content pages for each tab and add them to the stack
	for tab_name in tabs:
		row = Gtk.ListBoxRow()
		label = Gtk.Label.new(tab_name)
        # Add padding to the label inside the row
		label.set_margin_top(10)
		label.set_margin_bottom(10)
		row.set_child(label)
		sidebar_listbox.append(row)


        # Create a box for the content of this specific tab
		content_box = Gtk.Box.new(Gtk.Orientation.VERTICAL, 10)

		if tab_name == "VDPAU Information":
# ------- VDPAU information -------------------------------


			vdpauinfoColumnView = Gtk.ColumnView()
			vdpauinfoColumnView.props.show_row_separators = True
			vdpauinfoColumnView.props.show_column_separators = False
			
			factory_vdpau = Gtk.SignalListItemFactory()
			factory_vdpau.connect("setup",setup_expander)
			factory_vdpau.connect("bind",bind_expander)
			
			factory_vdpau_value = Gtk.SignalListItemFactory()
			factory_vdpau_value.connect("setup",setup)
			factory_vdpau_value.connect("bind",bind1)
			
			vdpauSelection = Gtk.SingleSelection()
			vdpauinfoStore = Gio.ListStore.new(ExpandDataObject)
			
			vdpauModel = Gtk.TreeListModel.new(vdpauinfoStore,False,True,add_tree_node)
			vdpauSelection.set_model(vdpauModel)
			
			vdpauinfoColumnView.set_model(vdpauSelection)
			
			vdpauColumnLhs = Gtk.ColumnViewColumn.new("VDPAU Information",factory_vdpau)
			vdpauColumnLhs.set_resizable(True)
			vdpauColumnRhs = Gtk.ColumnViewColumn.new("Details",factory_vdpau_value)
			vdpauColumnRhs.set_expand(True)

			
			vdpauinfoColumnView.append_column(vdpauColumnLhs)
			vdpauinfoColumnView.append_column(vdpauColumnRhs)

			
		#	vdpauinfoStore = Gtk.TreeStore(str,str,str)
		#	treeVdpauInfo = Gtk.TreeView.new_with_model(vdpauinfoStore)
		#	treeVdpauInfo.set_property("enable-grid-lines",1)

		#	setColumns(treeVdpauInfo,vdpauinfoTitle,300,0.0)

			vdpauinfoScrollbar = create_scrollbar(vdpauinfoColumnView)
			content_box.append(vdpauinfoScrollbar)

		elif tab_name == "Decoder Capabilities":
# ------- Decoder Capabilities ----------------------------
	
			decoderTab = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)

			decoderInfoColumnView = Gtk.ColumnView()
			decoderInfoColumnView.props.show_row_separators = True
			decoderInfoColumnView.props.show_column_separators = False
			
			factory_decoder = Gtk.SignalListItemFactory()
			factory_decoder.connect("setup",setup_expander)
			factory_decoder.connect("bind",bind_expander)
			
			factory_decoder_value1 = Gtk.SignalListItemFactory()
			factory_decoder_value1.connect("setup",setup)
			factory_decoder_value1.connect("bind",bind1)
			

			factory_decoder_value2 = Gtk.SignalListItemFactory()
			factory_decoder_value2.connect("setup",setup)
			factory_decoder_value2.connect("bind",bind2)


			factory_decoder_value3 = Gtk.SignalListItemFactory()
			factory_decoder_value3.connect("setup",setup)
			factory_decoder_value3.connect("bind",bind3)


			factory_decoder_value4 = Gtk.SignalListItemFactory()
			factory_decoder_value4.connect("setup",setup)
			factory_decoder_value4.connect("bind",bind4)

			decoderSelection = Gtk.SingleSelection()
			decoderStore = Gio.ListStore.new(ExpandDataObject2)
			filterSortDecoderStore = Gtk.FilterListModel(model=decoderStore)
			filter_decoder = Gtk.CustomFilter.new(_do_filter_decoder_view, filterSortDecoderStore)
			filterSortDecoderStore.set_filter(filter_decoder)
			
			vdpauModel = Gtk.TreeListModel.new(filterSortDecoderStore,False,True,add_tree_node2)
			decoderSelection.set_model(vdpauModel)
			
			decoderInfoColumnView.set_model(decoderSelection)
			
			decoderColumnLhs = Gtk.ColumnViewColumn.new("Decoder name",factory_decoder)
			decoderColumnLhs.set_resizable(True)
			decoderColumnRhs1 = Gtk.ColumnViewColumn.new("Level",factory_decoder_value1)
			decoderColumnRhs1.set_expand(True)
			decoderColumnRhs2 = Gtk.ColumnViewColumn.new("Macroblocks",factory_decoder_value2)
			decoderColumnRhs2.set_expand(True)
			decoderColumnRhs3 = Gtk.ColumnViewColumn.new("Width",factory_decoder_value3)
			decoderColumnRhs3.set_expand(True)
			decoderColumnRhs4 = Gtk.ColumnViewColumn.new("Height",factory_decoder_value4)
			decoderColumnRhs4.set_expand(True)
			
			decoderInfoColumnView.append_column(decoderColumnLhs)
			decoderInfoColumnView.append_column(decoderColumnRhs1)
			decoderInfoColumnView.append_column(decoderColumnRhs2)
			decoderInfoColumnView.append_column(decoderColumnRhs3)
			decoderInfoColumnView.append_column(decoderColumnRhs4)

		#	decoderStore = Gtk.ListStore(str,str,str,str,str,str)
		#	treeDecoder = Gtk.TreeView.new_with_model(decoderStore)
		#	treeDecoder.set_property("enable-grid-lines",1)

		#	setColumns(treeDecoder, decoderTitle, 300, 0.0)
			decoderSearchEntry = Gtk.SearchEntry()
			setMargin(decoderSearchEntry,0,5,0)
			decoderSearchEntry.set_property("placeholder_text","Type here to filter.....")
			decoderSearchEntry.connect("search-changed", _on_search_method_changed,filter_decoder)

			decoderScrollbar = create_scrollbar(decoderInfoColumnView)
			decoderTab.append(decoderScrollbar)
			decoderTab.append(decoderSearchEntry)


			content_box.append(decoderTab)

		elif tab_name == "Surface Limits":
			# -------- Surface Limits -----------------------------------

			vdpauSurfaceColumnView = Gtk.ColumnView()
			vdpauSurfaceColumnView.props.show_row_separators = True
			vdpauSurfaceColumnView.props.show_column_separators = False
			
			factory_surface_limits = Gtk.SignalListItemFactory()
			factory_surface_limits.connect("setup",setup_expander)
			factory_surface_limits.connect("bind",bind_expander)
			
			factory_surface_limits_value1 = Gtk.SignalListItemFactory()
			factory_surface_limits_value1.connect("setup",setup)
			factory_surface_limits_value1.connect("bind",bind1)
			

			factory_surface_limits_value2 = Gtk.SignalListItemFactory()
			factory_surface_limits_value2.connect("setup",setup)
			factory_surface_limits_value2.connect("bind",bind2)


			factory_surface_limits_value3 = Gtk.SignalListItemFactory()
			factory_surface_limits_value3.connect("setup",setup)
			factory_surface_limits_value3.connect("bind",bind3)


			vdpauSurfaceSelection = Gtk.SingleSelection()
			surfaceVideoStore = Gio.ListStore.new(ExpandDataObject2)
			
			vdpauSurfaceModel = Gtk.TreeListModel.new(surfaceVideoStore,False,True,add_tree_node2)
			vdpauSurfaceSelection.set_model(vdpauSurfaceModel)
			
			vdpauSurfaceColumnView.set_model(vdpauSurfaceSelection)
			
			vdpauSurfaceColumnLhs = Gtk.ColumnViewColumn.new("Suface",factory_surface_limits)
			vdpauSurfaceColumnLhs.set_resizable(True)
			vdpauSurfaceColumnLhs.set_expand(True)
			vdpauSurfaceColumnRhs1 = Gtk.ColumnViewColumn.new("Width",factory_surface_limits_value1)
			vdpauSurfaceColumnRhs1.set_expand(True)
			vdpauSurfaceColumnRhs2 = Gtk.ColumnViewColumn.new("Height",factory_surface_limits_value2)
			vdpauSurfaceColumnRhs2.set_expand(True)
			vdpauSurfaceColumnRhs3 = Gtk.ColumnViewColumn.new("Types",factory_surface_limits_value3)
			vdpauSurfaceColumnRhs3.set_expand(True)
			
			vdpauSurfaceColumnView.append_column(vdpauSurfaceColumnLhs)
			vdpauSurfaceColumnView.append_column(vdpauSurfaceColumnRhs1)
			vdpauSurfaceColumnView.append_column(vdpauSurfaceColumnRhs2)
			vdpauSurfaceColumnView.append_column(vdpauSurfaceColumnRhs3)


		#	surfaceVideoStore = Gtk.TreeStore(str,str,str,str,str)
		#	treeSurfaceVideoLimits = Gtk.TreeView.new_with_model(surfaceVideoStore)
		#	treeSurfaceVideoLimits.set_property("enable-grid-lines",1)

		#	setColumns(treeSurfaceVideoLimits,surfaceVideoTitle,300,0.0)

			surfaceVideoScrollbar = create_scrollbar(vdpauSurfaceColumnView)

			content_box.append(surfaceVideoScrollbar)
		elif tab_name == "Video Mixer":
			# -------- Video Mixer ---------------------------------------

			videoMixerColumnView = Gtk.ColumnView()
			videoMixerColumnView.props.show_row_separators = True
			videoMixerColumnView.props.show_column_separators = False

			factory_video_mixer = Gtk.SignalListItemFactory()
			factory_video_mixer.connect("setup",setup_expander)
			factory_video_mixer.connect("bind",bind_expander)

			factory_video_mixer_value = Gtk.SignalListItemFactory()
			factory_video_mixer_value.connect("setup",setup)
			factory_video_mixer_value.connect("bind",bind1)

			videoMixerSelection = Gtk.SingleSelection()
			videoMixerFeatureStore = Gio.ListStore.new(ExpandDataObject2)

			videoMixerModel = Gtk.TreeListModel.new(videoMixerFeatureStore,False,True,add_tree_node2)
			videoMixerSelection.set_model(videoMixerModel)

			videoMixerColumnView.set_model(videoMixerSelection)

			videoMixerColumnLhs = Gtk.ColumnViewColumn.new("Name",factory_video_mixer)
			videoMixerColumnLhs.set_resizable(True)
			videoMixerColumnRhs1 = Gtk.ColumnViewColumn.new("Supported",factory_video_mixer_value)
			videoMixerColumnRhs1.set_expand(True)

			videoMixerColumnView.append_column(videoMixerColumnLhs)
			videoMixerColumnView.append_column(videoMixerColumnRhs1)

			videoMixerFeatureScrollbar = create_scrollbar(videoMixerColumnView)

			content_box.append(videoMixerFeatureScrollbar)

		content_stack.add_titled(content_box, tab_name.replace(" ", "-").lower(), tab_name)

	sidebar_scrolled_window = Gtk.ScrolledWindow.new()
	sidebar_scrolled_window.set_child(sidebar_listbox)

    # Connect the listbox signal to change the stack page
	def on_row_activated(listbox, row):
		tab_label = row.get_child()
		tab_name = tab_label.get_label()
        
        # The name for the stack child is the tab name, with spaces replaced by dashes and lowercase
		child_name = tab_name.replace(" ", "-").lower()

        # Set the visible child of the stack using its name
		content_stack.set_visible_child_name(child_name)
        
	sidebar_listbox.connect("row-activated", on_row_activated)
	sidebar_listbox.select_row(sidebar_listbox.get_row_at_index(0))
#    content_stack.set_visible_child_name("System Information")
    # Set a minimum width for the sidebar


    # Create the Adw.NavigationPage wrappers for the sidebar and content
	sidebar_page = Adw.NavigationPage.new(sidebar_scrolled_window, "Sidebar")
	content_page = Adw.NavigationPage.new(content_stack, "Content")
    
    # Set the sidebar and content for the split view using the new pages
	split_view.set_sidebar(sidebar_page)
	split_view.set_content(content_page)
    
    # Add the split view to its new container
	split_view_container.append(split_view)

	decoderCapabilities()
	videoSurfaceLimits()
	VideoMixerFeature()
	vdpauInformation()

	tab2.append(split_view_container)

	return tab2
